/* ========================================================================== */
/*                                                                            */
/*   Filename.c                                                               */
/*   (c) 2012 Author                                                          */
/*                                                                            */
/*   Description                                                              */
/*                                                                            */
/* ========================================================================== */

/*
Method:
  1 - Calculation of STET Potential.
  2 - Calculation of STET acceleration.
  3 - Calculation of corresponding NEWTON matter density, 
                            using div(grad(a)) = 4 PI G rho.
                            (easier than FFT-1( FFT(StetPot)/FFT(InvDist) )) 
  4 - Plotting contours of this rho.

For printing contour with MAPLE :
  with(plots):
  contourplot(sin(x*y),x=-3..3,y=-3..3);

For printing contour with gnuplot :
  gnuplot> set samples 20
  gnuplot> set isosamples 21
  gnuplot> set xlabel "X axis"
  gnuplot> set ylabel "Y axis"
  gnuplot> set zlabel "Z axis" offset 1, 0
  gnuplot> set view 60, 30, 0.85, 1.1
  gnuplot> set title "3D gnuplot demo - contour plot"
  gnuplot> set contour
  gnuplot> splot x*y

  gnuplot> set cntrparam levels 20
  gnuplot> set title "3D gnuplot demo - contour plot (more contours)"
  gnuplot> replot
*/

/*  TEST EN FORMEL AVEC MAPLE (ne marche pas)
with(plots):
with(inttrans):
Digits := 50:

N := 5:
G := 0.67 * 10**(-11):    # A VERIFIER !!!!!!!!!!!!
A0 := 1:
A1 := 3:
A2 := 1:
A3 := 1:
x1 := 0:
y1 := 0:
x2 := 3:
y2 := 0:
x3 := -3:
y3 := 0:
F0 := A0:
AddedC := F0/6:

F1 := A1 * exp(-(x-x1)^2/2 - (y-y1)^2/2):
F2 := A2 * exp(-(x-x2)^2/2 - (y-y2)^2/2):
F3 := A3 * exp(-(x-x3)^2/2 - (y-y3)^2/2):
Rho := F1+F2+F3:

contourplot(Rho, x=-N..N, y=-N..N, contours=20):

StetCorr := (F0 + AddedC) / (F1 + AddedC):
Rho := F1+F2+F3:

FouRho := fourier(fourier(Rho, x, wx), y, wy):
InvDist := 1/(x^2+y^2):
FouInvDist := fourier(fourier(InvDist, x, wx), y, wy):
FouPot := G * FouRho * FouInvDist:
NewtonPot := invfourier(invfourier(FouPot, wx, x), wy, y):
StetPot := StetCorr * NewtonPot:

# FFT-1( FFT(StetPot)/FFT(InvDist) )) :
NewtonEquivPot := StetPot:
FouNewtonEquivPot := fourier(fourier(NewtonEquivPot, x, wx), y, wy):
FouNewtonEquivRho := FouNewtonEquivPot / (G * FouInvDist):
NewtonEquivRho := invfourier(invfourier(FouNewtonEquivRho, wx, x), wy, y):

contourplot(NewtonEquivRho, x=-N..N, y=-N..N, contours=20):

# Il n arrive pas a calculer ni fou(F1 + F2) ni fou de invdist.
# Simplifier F, puis essayer de calculer Potentiel par convolution.
# Pour cela il faut calculer fou de invdist.

contourplot(FouInvDist, x=-N..N, y=-N..N, contours=20):


#TEST 3D
with(plots):
with(inttrans):
Digits := 50:
InvDist := 1/(x^2+y^2+z^2):
FouInvDist := fourier(fourier(fourier(InvDist, x, wx), y, wy), z, wz);
wz=1:
plot(FouInvDist, wx=1..10, wy=1..10);

#TEST 2D +epsilon
with(plots):
with(inttrans):
Digits := 50:
Epsilon := 0.01:
n := 2 - Epsilon:
InvDist := 1/(x**n + y**n):
FouInvDist := fourier(fourier(InvDist, x, wx), y, wy);
wz=1:
plot(FouInvDist, wx=1..10, wy=1..10);

*/

/*  Simulating Bullet cluster.

  F1 = A1 * exp(-(x-x1)^2/2 - (y-y1)^2/2):
  F2 = A2 * exp(-(x-x2)^2/2 - (y-y2)^2/2):
  F3 = A3 * exp(-(x-x3)^2/2 - (y-y3)^2/2):
  RhoReal = F1+F2+F3:                            // Specific init.
  StetCorr = (F0 + AddedC) / (F1 + AddedC):      // Need code modif stet only. N = NIGM outside. Nmaxbullet = ? = NISM/10 !?
                                                 // NbPartStetOnly a rajouter provenant des premiers indices de particules.
                                                 // Il faudra verifier si c est stet simple ou stet complex qui est en jeu.
                                                 // Commencer avec stet simple.

  InvDist = 1/(x^2+y^2):                         // DONE
  NewtonPot = InvDist Convol RhoReal:            // DONE
  StetPot = StetCorr * NewtonPot:                // DONE. Executed : grid mode only, 1024 X 1024.
  RhoEquivNewton = Laplacien(StetPot)/(4*PI*G);  // To be coded. Easy. 
  RhoEquivNewtonContinue = denumerisation;       // To be coded. 
  gnuplotcontour(RhoEquivNewtonContinue);        // gnu plot: adding a new file, with a new set of options. OK. Quite easy now.
*/

// USAGE : GRID, or GRID3. 
// NbCell=1024 ... Perfs ... Caution, in gnuplot files, NbCell is divided by 2 for zooming(...). 
// INIT_TYPE = INIT_BULLET_CLUSTER     
// STET_MODE = STET_BULLET 

// NbCell = 4096 : plus d une nuit.

#include <stdlib.h> 
#include <string.h> 
#include <stdio.h>
#include <math.h>
#include "common.h"
#include "init.h"
#include "particles.h"
#include "hmi.h"
#include "stet.h"
#include "files_and_tools.h"
#include "potential.h"
#include "period.h"
#include "bulletcluster.h"

#if INIT_TYPE == INIT_BULLET_CLUSTER 

unsigned char BulletMode;
//double D0[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID];
//double D1[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID];
double CellNbPart[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID];
double CellNbPartStetGas[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID];
           // +2 for allowing the test of x, y, z out of bounds of regular grid.
double StetNbPart[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
double BulletWLRho[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID];
double BulletWLRhoSmooth[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID];
double BulletCenter1[2] = {BULLET_CENTER1_X, BULLET_CENTER1_Y};
double BulletCenter2[2] = {BULLET_CENTER2_X, BULLET_CENTER2_Y};
double BulletCenter3[2] = {BULLET_CENTER3_X, BULLET_CENTER3_Y};
double BulletCenter5[2] = {BULLET_CENTER5_X, BULLET_CENTER5_Y};
double BulletSigma1 = BULLET_SIGMA1;
double BulletSigma2 = BULLET_SIGMA2;
double BulletSigma3 = BULLET_SIGMA3;
double BulletSigma5 = BULLET_SIGMA5;
double RhoMax1 = BULLET_RHOMAX1;
double RhoMax2 = BULLET_RHOMAX2;
double RhoMax3 = BULLET_RHOMAX3;
double RhoMax4 = BULLET_RHOMAX4;
double RhoMax5 = BULLET_RHOMAX5;

char BulletDirName[BULLET_LGMAX_FILENAME];
char BulletCmdString[BULLET_LGMAX_FILENAME];
char BulletFileNameInitialMass[BULLET_LGMAX_FILENAME];
char BulletFileNamePotential[BULLET_LGMAX_FILENAME];
char BulletFileNamePotentialNewton[BULLET_LGMAX_FILENAME];
char BulletFileNameWLMass[BULLET_LGMAX_FILENAME];
char BulletFileNameWLMassNewton[BULLET_LGMAX_FILENAME];
char BulletFileName[BULLET_LGMAX_FILENAME];

double BulletGaussianRho( unsigned int jx, unsigned int jy, double Center[], double Sigma, double RhoMax)
{
  double Rho, dx, dy, SquareRay;
  dx = GRID_ORIGIN0 + jx * LGX_CELL - Center[0];
  dy = GRID_ORIGIN1 + jy * LGY_CELL - Center[1];
  SquareRay = dx*dx + dy*dy;
  Rho = (-SquareRay/(2*Sigma*Sigma));
  Rho = RhoMax * exp(-SquareRay/(2*Sigma*Sigma));
  return(Rho);
}

double BulletCos2Rho( unsigned int jx, unsigned int jy, double Center[], double Sigma, double RhoMax)
{
  double Rho, dx, dy, SquareRay, Ray, cosinus;
  dx = GRID_ORIGIN0 + jx * LGX_CELL - Center[0];
  dy = GRID_ORIGIN1 + jy * LGY_CELL - Center[1];
  SquareRay = dx*dx + dy*dy;
  Ray = sqrt(SquareRay);
  if(Ray <=  Sigma)
  {
    cosinus = cos( (PI/2) * Ray/Sigma);
    Rho = RhoMax * cosinus * cosinus;
  }
  else
  {
    Rho = 0;
  }
  return(Rho);
}

void InitBullet()    // Except stet init.
{
  unsigned int jx, jy, jz;
  double GridRay, dx, dy, SquareDistance, Rho, Rho1, Rho2, Rho3;    //, Rho4, Rho5;

  PartMass = MASS_MILKYWAY;    // PROVISORY.
  jz = 0;
  for(jy=0 ; jy<NBCELLY_GRID ; jy++)
  {
    for(jx=0 ; jx<NBCELLX_GRID ; jx++)
    {
      GridRay = (double) LG_GRID/2; 
      dx = jx * LGX_CELL - GridRay;
      dy = jy * LGY_CELL - GridRay;
      SquareDistance = dx*dx + dy*dy;
      if(SquareDistance >= GridRay * GridRay)
      {     // Cutting the corners of the squre for a homogenous calculation.
        CellNbPart[jx][jy][jz] = 0;
        CellNbPartStetGas[jx][jy][jz] = 0;
      }
      else
      {
        Rho1 = BulletGaussianRho( jx, jy, BulletCenter1, BulletSigma1, RhoMax1);
        Rho2 = BulletGaussianRho( jx, jy, BulletCenter2, BulletSigma2, RhoMax2);
        Rho3 = BulletGaussianRho( jx, jy, BulletCenter3, BulletSigma3, RhoMax3);
        Rho = Rho1 + Rho2 + Rho3;
        CellNbPart[jx][jy][jz] = Rho * BULLET_NBPART_CELL;
        CellNbPartStetGas[jx][jy][jz] = Rho1 * BULLET_NBPART_CELL;
      }
    }
  }
}

void InitStetBullet()
{
  #if GRAVIT_MODEL == STET 

//    StetSolSystContrib = 1;     // Fitted in order to have StetCorr max = 20, min between 1 and 5.
                                //  max value is fitted such as NbPartInStet=0 yields 20.
                                // min value must be fitted.
//  StetSolSystContrib = 20/(1.5E-2);   //10
//  StetSolSystContrib = 40/(1.5E-2);     //18

    StetSolSystContrib = 50/(1.5E-2);     // 20 : OK    FITTED
    StetAddedContrib = STET_ADDED_CONTRIB_RATIO_OVERSOLSYST * StetSolSystContrib;  // DONT TOUCH
  #endif 
}

double GetStetCorrBulletStet(double NbPartInStet, unsigned int ix, unsigned int iy, unsigned int iz) 
{
  #if GRAVIT_MODEL == STET 
    double StetCorr, Contrib;

    /* --- Calculation of StetCorr ---- */
    Contrib = NbPartInStet /PI;
    StetCorr  =  StetSolSystContrib + StetAddedContrib;      // 
    StetCorr /=  Contrib            + StetAddedContrib;      // 

    /* --- memorisation for debug ---- */
    StetCorrMemo[ix][iy][iz] = StetCorr;
    StetNbPart[ix][iy][iz] = NbPartInStet;  // DEBUG only.

    return(StetCorr);
  #else 
    return(1);     // For avoiding linker warnings
  #endif 
}

double GetStetCorrBulletNewton(double NbPartInStet, unsigned int ix, unsigned int iy, unsigned int iz) 
{
  return(1);    
}

double GetStetCorrBullet(double NbPartInStet, unsigned int ix, unsigned int iy, unsigned int iz) 
{
  if(BulletMode == BULLET_STET)
  {
    return(GetStetCorrBulletStet(NbPartInStet, ix, iy, iz));
  } 
  else
  {
    return(GetStetCorrBulletNewton(NbPartInStet, ix, iy, iz));
  } 
}

void WeakMassLensingCalculate()
{
  double Laplac, Laplac1, Laplac2, BulletWLRhoSmoothMin, BulletWLRhoSmoothMax;
  unsigned int jx, jy, jz, Firstx, Firsty, Lastx, Lasty;
  int i, j;
  
  // Calculate Laplacian of Potential
  for(jx=0 ; jx<NBCELLX_GRID ; jx++)
  {
    for(jy=0 ; jy<NBCELLY_GRID ; jy++)
    {
      jz = 0;
        Laplac1 =      Potential[jx+1][jy][jz];
        Laplac1 +=     Potential[jx-1][jy][jz];
        Laplac1 +=     Potential[jx][jy+1][jz];       
        Laplac1 +=     Potential[jx][jy-1][jz];
        Laplac1 -= 4 * Potential[jx][jy][jz];
        Laplac1 /= LGY_CELL * LGY_CELL;

        Laplac2 =  (1/2) * Potential[jx+1][jy+1][jz];
        Laplac2 += (1/2) * Potential[jx-1][jy-1][jz];
        Laplac2 += (1/2) * Potential[jx-1][jy+1][jz];       
        Laplac2 += (1/2) * Potential[jx+1][jy-1][jz];
        Laplac2 -=     2 * Potential[jx][jy][jz];
        Laplac2 /= LGY_CELL * LGY_CELL;

        Laplac = (1 - BULLET_GAMMA) * Laplac1 + BULLET_GAMMA * Laplac2;
        BulletWLRho[jx][jy][jz] = - Laplac / (4 * PI * G_CONST);
    }
  }

#define SMOOTHING
#ifdef SMOOTHING
  // Smoothing and rescaling
  for(jx=BULLET_INCR ; jx<NBCELLX_GRID-BULLET_INCR ; jx++)
  {
    for(jy=BULLET_INCR ; jy<NBCELLY_GRID-BULLET_INCR ; jy++)
    {
      jz=0;
      BulletWLRhoSmooth[jx][jy][jz] = 0;
      for(i=-BULLET_INCR ; i<=BULLET_INCR ; i++)
      {
        for(j=-BULLET_INCR ; j<=BULLET_INCR ; j++)
        {
          BulletWLRhoSmooth[jx][jy][jz] += BulletWLRho[jx+i][jy+j][jz];
        }
      }
      BulletWLRhoSmooth[jx][jy][jz] /= (2*BULLET_INCR+1) * (2*BULLET_INCR+1);
      BulletWLRhoSmooth[jx][jy][jz] *= BULLET_WLRHO_RESCALE;
    }
  }
#endif

  // Setting min=0
  Firstx = BULLET_CENTERX - BULLET_OFFSETX;
  Lastx = BULLET_CENTERX + BULLET_OFFSETX - 1;
  Firsty = BULLET_CENTERY - BULLET_OFFSETY;
  Lasty = BULLET_CENTERY + BULLET_OFFSETY - 1;
      //  BulletWLRhoSmoothMin = BulletWLRhoSmooth[Firstx][Firsty][0];
  BulletWLRhoSmoothMin = MAX_LONG_DOUBLE;
  for(jy=Firsty ; jy<Lasty ; jy++)
  {
    for(jx=Firstx ; jx<Lastx ; jx++)
    {
      jz=0;
      BulletWLRhoSmoothMin = min(BulletWLRhoSmoothMin, BulletWLRhoSmooth[jx][jy][jz]);
    }
  }
  for(jx=0 ; jx<NBCELLX_GRID ; jx++)
  {
    for(jy=0 ; jy<NBCELLY_GRID ; jy++)
    {
      jz=0;
      BulletWLRhoSmooth[jx][jy][jz] -= BulletWLRhoSmoothMin;
    }
  }

  // Rescaling again
  BulletWLRhoSmoothMax = 0;
  for(jy=Firsty ; jy<Lasty ; jy++)
  {
    for(jx=Firstx ; jx<Lastx ; jx++)
    {
      jz=0;
      BulletWLRhoSmoothMax = max(BulletWLRhoSmoothMax, BulletWLRhoSmooth[jx][jy][jz]);
    }
  }
  if(BulletWLRhoSmoothMax <= 0) Error(4321);
  for(jx=0 ; jx<NBCELLX_GRID ; jx++)
  {
    for(jy=0 ; jy<NBCELLY_GRID ; jy++)
    {
      jz=0;
      BulletWLRhoSmooth[jx][jy][jz] /= BulletWLRhoSmoothMax;
    }
  }
}
 
void PrintSimuInfoBullet(FILE *pFile)
{
  #if GRAVIT_MODEL == STET && STET_MODE != STET_SIMPLE
    fprintf( pFile, "# StetCorrMin=%f, StetCorrMax=%f \n", StetCorrMin, StetCorrMax);
  #endif
}

void CreateBulletFileFromCellNbPart(char *FileName, char *FileNameToBeCopied)
{
  FILE *pFile;
  unsigned jx, jy, jz, Firstx, Lastx, Firsty, Lasty;

  // Plotting WRho
  strcpy(FileName, ROOT_NAME);
  strcat(FileName, MODE_NAME);
  strcat(FileName, "\\");
  strcat(FileName, FileNameToBeCopied);
  strcpy(BulletFileName, FileName);
  strcat(BulletFileName, BULLET_GNUFILE_NAME_EXT);
  pFile = fopen(BulletFileName, "w");
  if (pFile == NULL) Error(1005);
  else;                                               

  PrintSimuInfoBullet(pFile);

  // Calculate mass density
  jz = 0;
  Firstx = BULLET_CENTERX - BULLET_OFFSETX;
  Lastx = BULLET_CENTERX + BULLET_OFFSETX - 1;
  Firsty = BULLET_CENTERY - BULLET_OFFSETY;
  Lasty = BULLET_CENTERY + BULLET_OFFSETY - 1;
  for(jy=Firsty ; jy<Lasty ; jy++)
  {
    for(jx=Firstx ; jx<Lastx ; jx++)
    {
      fprintf( pFile, "%f ", (double) CellNbPart[jx][jy][jz]);
    }
    fprintf( pFile, "\n");
  }

  fclose(pFile);
}

void CreateBulletFileFromBulletWLRho(char *FileName, char *FileNameToBeCopied)
{
  FILE *pFile;
  unsigned jx, jy, jz, Firstx, Lastx, Firsty, Lasty;

  // Plotting WRho
  strcpy(FileName, ROOT_NAME);
  strcat(FileName, MODE_NAME);
  strcat(FileName, "\\");
  strcat(FileName, FileNameToBeCopied);
  strcpy(BulletFileName, FileName);
  strcat(BulletFileName, BULLET_GNUFILE_NAME_EXT);
  pFile = fopen(BulletFileName, "w");
  if (pFile == NULL) Error(1006);
  else;                                               

  PrintSimuInfoBullet(pFile);

  // Calculate Laplacian of Potential
  jz = 0;
  Firstx = BULLET_CENTERX - BULLET_OFFSETX;
  Lastx = BULLET_CENTERX + BULLET_OFFSETX - 1;
  Firsty = BULLET_CENTERY - BULLET_OFFSETY;
  Lasty = BULLET_CENTERY + BULLET_OFFSETY - 1;
  for(jy=Firsty ; jy<Lasty ; jy++)
  {
    for(jx=Firstx ; jx<Lastx ; jx++)
    {
      fprintf( pFile, "%f ", BulletWLRhoSmooth[jx][jy][jz]);
    }
    fprintf( pFile, "\n");
  }

  fclose(pFile);
}

void CreateBulletFileFromBulletPotential(char *FileName, char *FileNameToBeCopied)
{
  FILE *pFile;
  unsigned jx, jy, jz, Firstx, Lastx, Firsty, Lasty;

  // Plotting WRho
  strcpy(FileName, ROOT_NAME);
  strcat(FileName, MODE_NAME);
  strcat(FileName, "\\");
  strcat(FileName, FileNameToBeCopied);
  strcpy(BulletFileName, FileName);
  strcat(BulletFileName, BULLET_GNUFILE_NAME_EXT);
  pFile = fopen(BulletFileName, "w");
  if (pFile == NULL) Error(1007);
  else;                                               

  PrintSimuInfoBullet(pFile);

  // Plot Potential
  jz = 0;
  Firstx = BULLET_CENTERX - BULLET_OFFSETX;
  Lastx = BULLET_CENTERX + BULLET_OFFSETX - 1;
  Firsty = BULLET_CENTERY - BULLET_OFFSETY;
  Lasty = BULLET_CENTERY + BULLET_OFFSETY - 1;
  for(jy=Firsty ; jy<Lasty ; jy++)
  {
    for(jx=Firstx ; jx<Lastx ; jx++)
    {
      fprintf( pFile, "%f ", Potential[jx][jy][jz]);
    }
    fprintf( pFile, "\n");
  }

  fclose(pFile);
}

void BulletPlot(char * FileName, char *FileNameToBeCopied)
{
      strcpy(BulletDirName, ROOT_NAME);            /* c:\ */
      strcat(BulletDirName, MODE_NAME);            //    s
      strcat(BulletDirName, "\\");                 //     \    
      strcpy(BulletCmdString, "cd ");                                         
      strcat(BulletCmdString, BulletDirName);
      system(BulletCmdString);                     // cd c:\s\   
      strcpy(BulletCmdString, "copy ");                                         
      strcat(BulletCmdString, FileName);
      strcat(BulletCmdString, ".txt");            
      strcat(BulletCmdString, " ");            
      strcat(BulletCmdString, BulletDirName);
      strcat(BulletCmdString, "tmp.txt");            
      system(BulletCmdString);                     // copy c:\s\name.txt c:\s\tmp.txt
      strcpy(BulletCmdString, "gnuplot.exe ");
      strcat(BulletCmdString, BulletDirName);
      strcat(BulletCmdString, BULLET_GNUEXEC_FILE_NAME); // gnuplot.exe c:\s\PlotGnuTmp3D.gnu
                             // Warning: current directory must be still available. Update it it is deleted....          
      system(BulletCmdString);                     // Plot of tmp.txt into tmp.png with correct options.
      strcpy(BulletCmdString, "del ");
      strcat(BulletCmdString, FileName);
      strcat(BulletCmdString, ".png");
      system(BulletCmdString);                     // del c:\s\name.png
      strcpy(BulletCmdString, "ren ");           
      strcat(BulletCmdString, BulletDirName);
      strcat(BulletCmdString, "tmp.png ");           
      strcat(BulletCmdString, FileNameToBeCopied);
      strcat(BulletCmdString, ".png");
      system(BulletCmdString);                     // ren c:\s\tmp.png name.png
      strcpy(BulletCmdString, "del ");           
      strcat(BulletCmdString, BulletDirName);
      strcat(BulletCmdString, "tmp.txt");           
      system(BulletCmdString);                     // del tmp.txt
}
void AddPotential_CLOSELargeCellBullet(unsigned int ix, unsigned int iy, unsigned int iz, int x, int y, int z, double *pNbPartInStet)
{
  double CellMass;
	int sx, sy, sz;
  unsigned int ax, ay, az;

  /************* Potential due to THIS large cell which is very close *****/
  for(sx = x - GRIDOPTIM_LARGECELL_RAY ; sx <= x + GRIDOPTIM_LARGECELL_RAY ; sx++)
  {
    for(sy = y - GRIDOPTIM_LARGECELL_RAY ; sy <= y + GRIDOPTIM_LARGECELL_RAY ; sy++)
    {
      #if (NB_DIM == 3)
        for(sz = z - GRIDOPTIM_LARGECELL_RAY ; sz <= z + GRIDOPTIM_LARGECELL_RAY ; sz++)
        {
      #else
        sz = 0;
      #endif
          if(IsValidCellBullet(sx, sy, sz))    
          {                         // If this cell is inside the disk.
            CellMass = CellNbPart[sx][sy][sz] * PartMass;
            ax = GridDistance(sx-ix); ay = GridDistance(sy-iy); az = GridDistance(sz-iz);
            Potential[ix][iy][iz] += CellMass * InvDist[ax][ay][az];
            #if GRAVIT_MODEL == STET        // For bullet here CellNbPartStetGas is used.
              if(InsideStetDiskBullet(ax, ay, az))
              {                    // Only for cell inside Stet disk.
                *pNbPartInStet += CellNbPartStetGas[sx][sy][sz]; 
              }                    // At the end nb of particles in Stet disk is memorized.
              else continue;       // Avoid : outside Stet disk.  
            #endif
          }
          else ;                   // Avoid if out of GridDisk.
      #if (NB_DIM == 3)
        }      // end for sz
      #endif
    }          // end for sy
  }            // end for sx
}           

double GetNbP_FARLargeCellBullet(int x, int y, int z)
{
  double NbP;

  /************* Potential due to THIS large cell NOT VERY close *****/
  NbP =   CellNbPart[x-1][y-1][z]        //  Calculate each small cell.
        + CellNbPart[x-1][y]  [z]        // Should use here
        + CellNbPart[x-1][y+1][z]        //  GridDistance
        + CellNbPart[x]  [y-1][z]        // but too slow and
        + CellNbPart[x]  [y]  [z]        // not important
        + CellNbPart[x]  [y+1][z]        // specially when
        + CellNbPart[x+1][y-1][z]        // NBCELLX_GRID is
        + CellNbPart[x+1][y]  [z]        // great.
        + CellNbPart[x+1][y+1][z];
  return(NbP);
}

#if MODE_CALCUL_POTENTIAL == MODE_GRID3

void CalculatePotentialCell3Bullet(unsigned int ix, unsigned int iy, unsigned int iz)
{
  double CellMass;
	int x, y, z, MinX, MaxX, MinY, MaxY, MinZ, MaxZ;
  unsigned int ax, ay, az;
  double NbP;
  double NbPartInStet = 0;
  #if GRAVIT_MODEL == STET
    double StetCorr;        // Corrective term added to Newton's acceleration.
  #endif

	Potential[ix][iy][iz] = 0;                      // And it will be added to this below.

  MinX = ix - (ix/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR;
  MinY = iy - (iy/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR;
  MinZ = iz - (iz/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR;
  MaxX = ix + ((IND_CELL_MAXX-ix)/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR;
  MaxY = iy + ((IND_CELL_MAXY-iy)/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR;
  MaxZ = iz + ((IND_CELL_MAXZ-iz)/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR;

	for(x = MinX ; x <= MaxX ; x+=NBCELL_OPTIM_INCR)
	{
		for(y = MinY ; y <= MaxY ; y+=NBCELL_OPTIM_INCR)
		{                                       // for jx=jy=0, Potential Potential[jx][jy] = 0
      #if (NB_DIM == 3)
        for(z = MinZ ; z <= MaxZ ; z+=NBCELL_OPTIM_INCR)
        { 
      #else
          z = 0;
      #endif
          ax =  GridDistance(x-ix); ay =  GridDistance(y-iy); az =  GridDistance(z-iz);

          if(IsValidBigCellBullet(x, y, z))    // Is x y z is in the grid, with a possible calculation ?
          {                         // If the center of this large cell is inside the disk.
            if(GridOptimSquare[ax][ay][az])    // If this is a very close large square cell.
            {                                             // Large cell close to us.
              /* ------------- Large cell CLOSE : NO optimisation --------------------------- */
              AddPotential_CLOSELargeCellBullet(ix, iy, iz, x, y, z, &NbPartInStet);
                               // Counting of particles in STET disk is done in this function.
            }
            else               // else of: if(GridOptimSquare[ax][ay][az])  
            {                                   // Not VERY CLOSE:
              if(GridOptimDisk[ax][ay][az])     // Far but NOT VERY far:
              {                        // Inside optimizing disk : jx+jy not very strong.

                /* ------------- Large cell FAR : slight optimisation ----------------------- */
                NbP = GetNbP_FARLargeCellBullet(x, y, z);
              }                                          
              else                     // else: GridOptimDisk[ax][ay][az] = 0
              { 

                /* ------------- Large cell VERY FAR : STRONG and dangerous optimisation ---- */
                NbP = CellNbPart[x][y][z] * NBSMALLCELL_IN_LARGECELL; 
                                       // Only centered small cell is read for this large cell. 
              }                                        // end of if GridOptimDisk[ax][ay][az]

              /* ---- Updating potential because of this large cell NOT close --------------- */
              CellMass = NbP * PartMass;
              Potential[ix][iy][iz] += CellMass * InvDist[ax][ay][az];
            }                        // end of: if(GridOptimSquare[ax][ay][az])
          }                          // end of  if(IsValidBigCell(x, y, z)
      #if (NB_DIM == 3)
        }     // end for z
      #endif
		}         // end for y
	}           // end for x

  #if GRAVIT_MODEL == STET
    StetCorr = GetStetCorrBullet(NbPartInStet, ix, iy, iz);
    StetCorrMax = max(StetCorr, StetCorrMax);
    StetCorrMin = min(StetCorr, StetCorrMin);
    Potential[ix][iy][iz] *= StetCorr;
       //NbPartInStetDebug[ix][iy][iz] = NbPartInStet;   // DEBUG ONLY !
  #endif
}
#else
  void CalculatePotentialCell3Bullet(unsigned int ix, unsigned int iy, unsigned int iz) {}
#endif                       // end of #if MODE_CALCUL_POTENTIAL == MODE_GRID3.

void CalculatePotentialCellBullet(unsigned int ix, unsigned int iy, unsigned int iz)
{
	int x, y, z;
  unsigned int ax, ay, az;
  double NbP;
  #if GRAVIT_MODEL == STET
    double StetCorr;        // Corrective term added to Newton's acceleration.
    double NbPartInStet = 0.;
  #endif

	Potential[ix][iy][iz] = 0;                      // And it will be added to this below.

	for(x = 0 ; x <= IND_CELL_MAXX ; x++)
	{
		for(y = 0 ; y <= IND_CELL_MAXY ; y++)
		{                                       // for jx=jy=0, Potential Potential[jx][jy] = 0
      #if (NB_DIM == 3)
        for(z = 0 ; z <= IND_CELL_MAXZ ; z++)
        { 
      #else
          z = 0;
      #endif
          if(IsValidCellBullet(x, y, z))  // Is x y z is in grid disk, with possible calculation ?
          {                            // If the center of this cell is inside the disk.
            ax =  GridDistance(x-ix); ay =  GridDistance(y-iy); az =  GridDistance(z-iz);
            NbP = CellNbPart[x][y][z]; 
            Potential[ix][iy][iz] += NbP * PartMass * InvDist[ax][ay][az];

            /* ---- Counting number of particles inside STET disk ------------------------- */
            #if GRAVIT_MODEL == STET                   // For this large cell NOT close to us.
              if(InsideStetDiskBullet(ax, ay, az))
              {                    // Only for cell inside Stet disk.
                NbPartInStet += CellNbPartStetGas[x][y][z];
              }                    // At the end nb of all particles in Stet disk is memorized.
              else ;               // Avoid : outside Stet disk.  
            #endif
          }
          else ;                       // Out of the grid disk. 
      #if (NB_DIM == 3)
        }     // end for z
      #endif
		}         // end for y
	}           // end for x

  #if GRAVIT_MODEL == STET
    StetCorr = GetStetCorrBullet(NbPartInStet, ix, iy, iz);
    StetCorrMax = max(StetCorr, StetCorrMax);
    StetCorrMin = min(StetCorr, StetCorrMin);
    Potential[ix][iy][iz] *= StetCorr;
      //NbPartInStetDebug[ix][iy][iz] = NbPartInStet;   // DEBUG ONLY !
  #endif
}

void GravitPotentialCalculateByGridBullet()
{
  int ix;                   // Signed because of parallel need.

  /* ---------------- Parallelisation on multi-core --------------------- */
  #ifdef _OPENMP          // Noticeably avoid parallelisation with STEP by STEP.
    #pragma omp parallel for private( ix )   // Warning doesnt works with step by step. 
  #endif

  /* ---------------- For each cell in the grid calculate its potential - */
  for(ix=0 ; ix<NBCELLX_GRID ; ix++)
  {
    unsigned int iy, iz;
    for(iy=0 ; iy<NBCELLY_GRID ; iy++)
    {
      for(iz=0 ; iz<NBCELLZ_GRID ; iz++)
      {
        if(PotentialDisk[ix][iy][iz])
        {
          switch(MODE_CALCUL_POTENTIAL)
          {
            case MODE_GRID:          // Calculate the integral convolution step by step.
              CalculatePotentialCellBullet(ix, iy, iz); 
              break;
            case MODE_GRID3:         // Calcul integral convol step by step optimized manner.
              CalculatePotentialCell3Bullet(ix, iy, iz);
              break;
          }
        }else ;     // No need, out of disk (positout): continue
      }
    }
  }
}

void GravitPotentialCalculateBullet()
{
  GravitPotentialCalculateByGridBullet();   // No FFT today because of poor precision FFT available libs.
}

void BulletCalc()
{
  unsigned char tmp;
  
	fprintf(stdout," ----------------- DEBUT -------------- \n");

  /************** Initialisation of modules *****************************/
	fprintf(stdout," ----------------- Debut des inits -------------- \n");
  InitPotential();          // Init of potential.c module.
  InitBullet();              // Init of bullet cluster matter. 
  InitStet();               // Init of stet.h module. Executed after Init().
       //	InitHmi();                // Init of hmi.c module. For debug.
	fprintf(stdout," ----------------- Fin des inits -------------- \n");

        //TestBullet();     // TESTS

  /************** Plotting initial cluster for DEBUG ********************/
     //  PlotOpenCloseFileIfNeeded(0);         // Open "debug" files.
     //  PlotPartsOneStep(0);                  // Print posold for each particle.

  /************** Plotting initial mass *********************************/
	fprintf(stdout," ----------------- Initial mass -------------- \n");
  CreateBulletFileFromCellNbPart(BulletFileNameInitialMass, BULLET_INITIALMASS_GNUFILE_NAME_WITHOUTEXT);
  BulletPlot(BulletFileNameInitialMass, BULLET_INITIALMASS_GNUFILE_NAME_WITHOUTEXT);

	fprintf(stdout," ----------------- STET -------------- \n");
  BulletMode = BULLET_STET;

  /************** Calculation of potential ******************************/
	fprintf(stdout," ----------------- Calculating potential -------------- \n");
  GravitPotentialCalculateBullet();   // Calculate gravitational potential.
                                      // Here are perf optimisations.
  /************** Plotting potential ************************************/
	fprintf(stdout," ----------------- Potential -------------- \n");
  CreateBulletFileFromBulletPotential(BulletFileNamePotential, BULLET_POTENTIAL_GNUFILE_NAME_WITHOUTEXT);
  BulletPlot(BulletFileNamePotential, BULLET_POTENTIAL_GNUFILE_NAME_WITHOUTEXT);

  /************** Calculation and plotting weak mass lensing ************/
	fprintf(stdout," ----------------- Weak lensing mass -------------- \n");
  WeakMassLensingCalculate();             // Calculatig BulletWLRho array.
  CreateBulletFileFromBulletWLRho(BulletFileNameWLMass, BULLET_WLMASS_GNUFILE_NAME_WITHOUTEXT);
  BulletPlot(BulletFileNameWLMass, BULLET_WLMASS_GNUFILE_NAME_WITHOUTEXT);

	fprintf(stdout," ----------------- NEWTON -------------- \n");
  BulletMode = BULLET_NEWTON;

  /************** Calculation of potential ******************************/
	fprintf(stdout," ----------------- Calculating potential -------------- \n");
  GravitPotentialCalculateBullet();   // Calculate gravitational potential.
                                      // Here are perf optimisations.
  /************** Plotting potential ************************************/
	fprintf(stdout," ----------------- Potential -------------- \n");
  CreateBulletFileFromBulletPotential(BulletFileNamePotentialNewton, BULLET_POTENTIAL_GNUFILE_NAME_WITHOUTEXT_NEWTON);
  BulletPlot(BulletFileNamePotentialNewton, BULLET_POTENTIAL_GNUFILE_NAME_WITHOUTEXT_NEWTON);

  /************** Calculation and plotting weak mass lensing ************/
	fprintf(stdout," ----------------- Weak lensing mass -------------- \n");
  WeakMassLensingCalculate();             // Calculatig BulletWLRho array.
  CreateBulletFileFromBulletWLRho(BulletFileNameWLMassNewton, BULLET_WLMASS_GNUFILE_NAME_WITHOUTEXT_NEWTON);
  BulletPlot(BulletFileNameWLMassNewton, BULLET_WLMASS_GNUFILE_NAME_WITHOUTEXT_NEWTON);

  /**************** Simulation ending if ever needed ********************/
         //	PlotEndFile();                         // Close "debug" files.        
	printf(" ----------------- FIN -------------- \n");
//  scanf("%d", &tmp);

}

/* ----------- TESTs and DEBUG ---------- */
double tmpBullet;

void TestBullet()
{
//  InitStetBullet();
  tmpBullet = GetStetCorrBullet(0, 0, 0, 0); 
}
/* ----------- END of TESTs and DEBUG ---------- */

#endif    // #if INIT_TYPE == INIT_BULLET_CLUSTER 

