/*************************************** SIMULATION GENERAL SHAPE *******************************************/
#define NB_DIM              2
#define NB_STEP				      2000000000   // For the 2nd phase, after cooling.    
//#define NB_STEP_FIRSTPHASE  300   // For the 1rst phase, before cooling.
#define NB_STEP_FIRSTPHASE  2000000000   // For the 1rst phase, before cooling.
//#define INCR_SIMU           100
#define INCR_SIMU           2000000000
#define NB_STEP_SECONDPHASE 3300   // For the 1rst phase, before cooling.
#define INCR_SIMU2          250

/* --------------------------------------- GRAVITATIONAL MODELs -------------------------------------------- */
#define NEWTON            0
#define STET              1
#define TELT              2
#define GRAVIT_MODEL      STET                                   // TOUCH THIS ONE.

/********************************************* MODE DEBUG or RELEASE *****************************************/
#define DEBUG_NORM        0          // NO parallelis for correct step by step debug. HMI output in debug dir.
#define DEBUG_SOLSYST     1          // Parallelisation. HMI output in debug directories.
#define RELEASE           2          // Parallelisation. HMI output in non debug directories.
#define MODE_DEBUG        RELEASE                              // TOUCH THIS ONE.

/********************************************* GENERAL CONSTANTS *********************************************/
#define PI				        3.14159
#define LN_2              0.69314718055994        //
#define MIN			          60                      // 1 min in sec.
#define HOUR              3600                    // 1 hour in sec.
#define DAY				        (24*HOUR)               // 1 day in hour.
#define MONTH				      (30*DAY)                // Roughly.
#define YEAR			        (365*DAY)               
#define PARSEC			      3.08E16               
#define KILPARSEC		      (1000*PARSEC)
#define MEGAPARSEC		    (1000*KILPARSEC)
#define CM		            0.01
#define CM3		            (CM*CM*CM)
#define G_CONST           6.673E-11               // G gravitational constant.
#define MAX_LONG_DOUBLE   1.E307                  // Max value for a long double.
#define MAX_UNSIGNED_INT  (1.E32-1)               // Max value for an unsigned int.


/* ---------------------------------------- Solar system ---------------------------------------- */
#define MASS_SUN		      2E30
#define EARTH_SUN_DIST    1.496E11                // 1.4960  10 11 m
#define V_EARTH           (2 * PI * EARTH_SUN_DIST / YEAR)

/* ----------------------------------------- Cosmology ------------------------------------------ */
#define MASS_MILKYWAY	      (5.8E11 * MASS_SUN)   // Goes with IR_MAX = 8, fitted.
#define MASS_SAGITA		      (4.31E6 * MASS_SUN)   // OK so ratio of E5 which is out of nb of simul particles.
#define R_MILKYWAY          (17 * KILPARSEC)
#define DIAM_MILKYWAY       (2 * R_MILKYWAY)
#define REVOL_TIME_MILKYWAY (2E8*YEAR)            // Real galactic revolution time.
#define V_STARS_MILKYWAY    230000                // rougly 230 km/s for Milky-way stars.
#define WIDTH_MILKYWAY      (0.6 * KILPARSEC)
       // END OF THE GALAXY : Omega = V_STARS_MILKYWAY / R_MILKYWAY =  1,2 E-2 BY-1, T = 2 pi/omega.
                           // T = 520 BY with V=200000.
                           // T = 416 BY with V=250000.   Triassic = 200 to 250 BY.
       // SUN PATH : T = (8,3 kpc/17 kpc)520 = 253 BY with V=200000.
                  // T = 203 BY with V=250000.   Triassic = 200 to 250 BY.
#define SIGMA_EXT           1.E-28                    // 1/m3  (E-28 m-3 = 1 barn)
#define RHO_C               1.E-26                          // kg/m3
#define RHO_U               (0.05 * RHO_C)
//#define RHO_GAL_AROUND_SUN  0.708E-20                 // kg/m3
//#define RHO_GAL_AROUND_SUN  (0.05 * 0.708E-20)                 // kg/m3                PROVISOIRE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//#define RHO_GAL_AROUND_SUN  (2.5E-20)                 // kg/m3                PROVISOIRE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//#define RHO_GAL_AROUND_SUN  (0.05 * 0.708E-20)     //PROVISOIRE !!!!!!!!!!!!!!!  RecupsimuOK
#define RHO_GAL_AROUND_SUN  0.708E-20
#define NO_GAL_AROUND_SUN   (0.67E4/CM3)              // in m-3
#define RHO_IGM             (4 * RHO_U)
#define N_IGM               (1.5E-3/CM3)
#define OMEGA_UNIVERSE_MATDENS 0.05                   // RhoUniverse / RhoCritical
#define IGM_RELAT_MATDENS      10                     // RhoIgm / RhoUniverse.      

extern long int StepNum;   // StepNum. Here for debug and tracking precision and time.
extern int totaltime;
extern void Error(int ErrNum);
extern double CalculTime(double total_time);
