/*   CALCULATION of potential with FFT (fastest way).
     ... But a huge precision issue on Windows 32 bits make it impossible 
     to use today with fftw library. Try to find another fft lib or execute 
     this fftw lib but on Windows 64 bits ...
*/

#include <stdlib.h> 
#include <string.h> 
#include <stdio.h>
#include <math.h>
#include <fftw3.h>
#include <complex.h>
#ifdef _OPENMP          
#include <omp.h>
#endif

#include "common.h"
#include "hmi.h"
#include "init.h"
#include "particles.h"
#include "potential.h"
#include "fft.h"

#if MODE_CALCUL_POTENTIAL==MODE_FFT

double CellMass[NBCELL_FFT];           // TO BE OPTIMISED FOR MEMORY !!!!!
double InvDistFft[NBCELL_FFT];
double PotentialFft[NBCELL_FFT];
fftw_complex CellMassFreq[NBCELL_FFT];
fftw_complex InvDistFreq[NBCELL_FFT];
fftw_complex PotentialFreq[NBCELL_FFT];
double_complex CellMassFreqPerso[NBCELL_FFT];
double_complex PotentialFreqPerso[NBCELL_FFT];
double_complex InvDistFreqPerso[NBCELL_FFT];
fftw_plan CellMassFftPlan;
fftw_plan PotInvFftPlan;

/* --------------------- FFT calculations INIT -------------------- */
void InitFFT()
{
	unsigned int jx,jy;
	fftw_plan FftPlan;
	int i;

	memset(InvDistFft,0,sizeof(InvDistFft));
	memset(PotentialFft,0,sizeof(PotentialFft));
  memset(CellMass, 0, sizeof(CellMass));

	////////////////////////////////// DEBUG !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	//  memset(InvDist, 0, sizeof(InvDist));
	//  InvDist[0][0] = 1;   // DIRAC 0  
	////////////////////////////////// DEBUG !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	/* ---- Prepare plan FFT of InverseDistance array --------------------- */
	FftPlan = fftw_plan_dft_r2c_2d(NBCELLX_FFT,NBCELLY_FFT,InvDistFft,InvDistFreq,FFT_PLANNING_OPTION);
	// InvDistFft has been erased here.
	/* --- Put in shape for FFT ---- */
	for(jy=0 ; jy<NBCELLY_FFT ; jy++)
	{
		for(jx=0 ; jx<NBCELLX_FFT ; jx++)
		{
			i = jx + jy*NBCELLX_FFT;
			InvDistFft[i] = InvDist[jx][jy];
		}
	}

	/* ---- Construct the 3 other quarters by central symmetry ---------- */
	FillBySymmetryDouble(InvDistFft);

	/* ---- Execute FFT of InverseDistance array ------------------------ */
	fftw_execute(FftPlan);                      // InvDistFft -> InvDistFreq

	/* ---- Put the result in a readable format ------------------------ */
	memcpy(InvDistFreqPerso,InvDistFreq,sizeof(InvDistFreqPerso));
	// This will be used as such in the multiplication in freq space.
	/* ---- Plan FFT of CellMass array --------------------------------- */
	CellMassFftPlan = fftw_plan_dft_r2c_2d(NBCELLX_FFT,NBCELLY_FFT,
										   CellMass,CellMassFreq,FFT_PLANNING_OPTION);
	// CellMass has been erased here.
	memset(CellMass,0,sizeof(CellMass));

	/* ---- Plan INVERSE FFT of Product (potential)  ------------------- */
	PotInvFftPlan = fftw_plan_dft_c2r_2d(NBCELLX_FFT,NBCELLY_FFT,
										 PotentialFreq,PotentialFft,FFT_PLANNING_OPTION);
	// PotentialFreq has been erased here.
	memset(PotentialFreq,0,sizeof(PotentialFreq));
}

/* --- Fillling the 3 remaining quarters of an array by 2D parity */
void FillBySymmetry(double_complex a[])
{
	unsigned int jx,jy,kx,ky;
	unsigned int i,k;

	/* --- Fillling the 3 remaining quarters by 2D pariy ---------- */
	for(jy=1 ; jy<=NBCELLY_GRID ; jy++)
	{
		for(jx=1 ; jx<=NBCELLX_GRID ; jx++)
		{
			i = jx + jy*NBCELLX_FFT;        // Flat indice in large array.

			/* --- Initialize now the 3 other quarters of large array --- */
			kx = jx;                           ky = NBCELLY_FFT - jy;
			k = kx + ky*NBCELLX_FFT;            // By central symmetry.
			a[k] = a[i];

			kx = NBCELLX_FFT - jx;             ky = jy;
			k = kx + ky*NBCELLX_FFT;
			a[k] = a[i];

			kx = NBCELLX_FFT - jx;             ky = NBCELLY_FFT - jy;
			k = kx + ky*NBCELLX_FFT;
			a[k] = a[i];
		}
	}
	for(jy=1 ; jy<NBCELLY_GRID ; jy++)
	{
		i = jy*NBCELLX_FFT;
		ky = NBCELLY_FFT - jy;
		k = ky*NBCELLX_FFT;            // By central symmetry.
		a[k] = a[i];
	}
	for(jx=1 ; jx<NBCELLX_GRID ; jx++)
	{
		kx = NBCELLX_FFT - jx;
		a[kx] = a[jx];
	}
}

/* --- Same function but with a double typed array ----- */
void FillBySymmetryDouble(double a[])
{
	unsigned int jx,jy,kx,ky;
	unsigned int i,k;

	/* --- Fillling the 3 remaining quarters by 2D pariy ---------- */
	for(jy=1 ; jy<=NBCELLY_GRID ; jy++)
	{
		for(jx=1 ; jx<=NBCELLX_GRID ; jx++)
		{
			i = jx + jy*NBCELLX_FFT;        // Flat indice in large array.

			/* --- Initialize now the 3 other quarters of large array --- */
			kx = jx;                           ky = NBCELLY_FFT - jy;
			k = kx + ky*NBCELLX_FFT;            // By central symmetry.
			a[k] = a[i];

			kx = NBCELLX_FFT - jx;             ky = jy;
			k = kx + ky*NBCELLX_FFT;
			a[k] = a[i];

			kx = NBCELLX_FFT - jx;             ky = NBCELLY_FFT - jy;
			k = kx + ky*NBCELLX_FFT;
			a[k] = a[i];
		}
	}
	for(jy=1 ; jy<NBCELLY_GRID ; jy++)
	{
		i = jy*NBCELLX_FFT;
		ky = NBCELLY_FFT - jy;
		k = ky*NBCELLX_FFT;            // By central symmetry.
		a[k] = a[i];
	}
	for(jx=1 ; jx<NBCELLX_GRID ; jx++)
	{
		kx = NBCELLX_FFT - jx;
		a[kx] = a[jx];
	}
}

void GravitPotentialCalculateByFFT()
{
	unsigned int jx,jy;
	int i;

	////////////////////////////////// DEBUG !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	//  memset(CellNbPart, 0, sizeof(CellNbPart));
	//  CellNbPart[0][0] = 1;   // DIRAC 0  
	//  CellNbPart[0][1] = 0;   // DIRAC 0  
	//  CellNbPart[0][2] = 1;   // DIRAC 0  
	//  CellNbPart[1][0] = 1;   // DIRAC 0  
	////////////////////////////////// DEBUG !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	/* ---- Construct MassCell array in shape for FFT functions -------- */
	for(jy=0 ; jy<NBCELLY_FFT ; jy++)
	{
		for(jx=0 ; jx<NBCELLX_FFT ; jx++)
		{
			i = jx + jy*NBCELLX_FFT;
			////////////////////////////////// DEBUG !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			CellMass[i] = CellNbPart[jx][jy] * PartMass;  // TO BE OPTIMISED LATER ON calculation on one quarter only.
			//      CellMass[i] = CellNbPart[jx][jy];
			////////////////////////////////// DEBUG !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		}
	}

	/* ---- Execute FFT of CellMass array ------------------------------ */
	fftw_execute(CellMassFftPlan);             // CellMass -> CellMassFreq

	/* ---- Multiply CellMass in freq with inverse function in freq  --- */
	memcpy(CellMassFreqPerso,CellMassFreq,sizeof(CellMassFreqPerso));
	for(jy=0 ; jy<NBCELLY_FFT ; jy++)
	{
		for(jx=0 ; jx<NBCELLX_FFT ; jx++)
		{
			i = jx + jy*NBCELLX_FFT;
			PotentialFreqPerso[i] = mul_real_complex(InvDistFreqPerso[i],CellMassFreqPerso[i]);
		}
	}

	////////////////////////////////// DEBUG !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	//memcpy(PotentialFreqPerso, CellMassFreq, sizeof(PotentialFreqPerso)); 
	////////////////////////////////// DEBUG !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	/* --- Fillling the 3 remaining quarters by 2D parity ---------- */
	FillBySymmetry(PotentialFreqPerso);

	memcpy(PotentialFreq,PotentialFreqPerso,sizeof(PotentialFreq));
	// Type conversion. TO BE OPTIMISED LATER ON.

	/* ---- Execute INVERSE FFT of Product  ---------------------------- */
	fftw_execute(PotInvFftPlan); // PotentialFreq -> PotentialFft. Result OK but FOIS 25

	/* ---- Put Potential in normal shape from FFT potential ----------- */
	for(jy=0 ; jy<NBCELLY_FFT ; jy++)
	{
		for(jx=0 ; jx<NBCELLX_FFT ; jx++)
		{
			i = jx + jy*NBCELLX_FFT;
			Potential[jx][jy] = PotentialFft[i] / NBCELL_FFT;
		}
	}
}

#else
  extern void InitFFT(){}
  extern void GravitPotentialCalculateByFFT(){}
#endif
