/*       HUMAN MAN INTERFACE : FILES and TOOLS initialisation 
*/

#include <stdio.h>
#include "common.h"
#include "hmi.h"
#include "init.h"
#include "particles.h"
#include "stet.h"
#include "files_and_tools.h"
#include "potential.h"
#include "fft.h"

FilesToPlot_t FileSet[NB_FILE_SET] =
{
	{
		NB_STEP_IN_SFILE,                 // A 0 here implies the end of this used array.
		NB_PRINT_IN_SFILE,
		PRINT_SDELTA,
    NB_SFILE_MAX,
    SFILE_TYPE,
		"Small"
	},
	{
		NB_STEP_IN_MFILE,                 // A 0 here implies the end of this used array.
		NB_PRINT_IN_MFILE,
		PRINT_MDELTA,
    NB_MFILE_MAX,
    MFILE_TYPE,
		"Medium"
	},
	{
		NB_STEP_IN_BFILE,
		NB_PRINT_IN_BFILE,
		PRINT_BDELTA,
    NB_BFILE_MAX,
    BFILE_TYPE,
		"Big"
	},
  {
    NB_STEP_IN_VBFILE,
    NB_PRINT_IN_VBFILE,
    PRINT_VBDELTA,
    NB_VBFILE_MAX,
    VBFILE_TYPE,
    "VBig"
  },
  {
		NB_STEP_IN_GSFILE,                 // A 0 here implies the end of this used array.
		NB_PRINT_IN_GSFILE,
		PRINT_GSDELTA,
    NB_GSFILE_MAX,
    GSFILE_TYPE,
		"Small"
	},
	{
		NB_STEP_IN_GMFILE,                 // A 0 here implies the end of this used array.
		NB_PRINT_IN_GMFILE,
		PRINT_GMDELTA,
    NB_GMFILE_MAX,
    GMFILE_TYPE,
		"Medium"
	},
	{
		NB_STEP_IN_GBFILE,
		NB_PRINT_IN_GBFILE,
		PRINT_GBDELTA,
    NB_GBFILE_MAX,
    GBFILE_TYPE,
		"Big"
	},
  {
    NB_STEP_IN_GVBFILE,
    NB_PRINT_IN_GVBFILE,
    PRINT_GVBDELTA,
    NB_GVBFILE_MAX,
    GVBFILE_TYPE,
    "VBig"
  },
  {
    NB_STEP_IN_SDRSFILE,
    NB_PRINT_IN_SDRSFILE,
    PRINT_SDRSDELTA,
    NB_SDRSFILE_MAX,
    SDRSFILE_TYPE,
    "SDRSmall"
  },
  {
    NB_STEP_IN_SDRMFILE,
    NB_PRINT_IN_SDRMFILE,
    PRINT_SDRMDELTA,
    NB_SDRMFILE_MAX,
    SDRMFILE_TYPE,
    "SDRMedium"
  },
  {
    NB_STEP_IN_SDTSFILE,
    NB_PRINT_IN_SDTSFILE,
    PRINT_SDTSDELTA,
    NB_SDTSFILE_MAX,
    SDTSFILE_TYPE,
    "SDTSmall"
  },
  {
    NB_STEP_IN_SDTMFILE,
    NB_PRINT_IN_SDTMFILE,
    PRINT_SDTMDELTA,
    NB_SDTMFILE_MAX,
    SDTMFILE_TYPE,
    "SDTMedium"
  },
  {
    NB_STEP_IN_SIGRSFILE,
    NB_PRINT_IN_SIGRSFILE,
    PRINT_SIGRSDELTA,
    NB_SIGRSFILE_MAX,
    SIGRSFILE_TYPE,
    "SIGRSmall"
  },
  {
    NB_STEP_IN_SIGRMFILE,
    NB_PRINT_IN_SIGRMFILE,
    PRINT_SIGRMDELTA,
    NB_SIGRMFILE_MAX,
    SIGRMFILE_TYPE,
    "SIGRMedium"
  },
  {
    NB_STEP_IN_SIGTSFILE,
    NB_PRINT_IN_SIGTSFILE,
    PRINT_SIGTSDELTA,
    NB_SIGTSFILE_MAX,
    SIGTSFILE_TYPE,
    "SIGTSmall"
  },
  {
    NB_STEP_IN_SIGTMFILE,
    NB_PRINT_IN_SIGTMFILE,
    PRINT_SIGTMDELTA,
    NB_SIGTMFILE_MAX,
    SIGTMFILE_TYPE,
    "SIGTMedium"
  },
  {
    NB_STEP_IN_MEANT0FILE,
    NB_PRINT_IN_MEANT0FILE,
    PRINT_MEANT0DELTA,
    NB_MEANT0FILE_MAX,
    MEANT0FILE_TYPE,
    "MEANT0"
  },
  {
    NB_STEP_IN_MEANTSFILE,
    NB_PRINT_IN_MEANTSFILE,
    PRINT_MEANTSDELTA,
    NB_MEANTSFILE_MAX,
    MEANTSFILE_TYPE,
    "MEANTSmall"
  },
  {
    NB_STEP_IN_MEANTMFILE,
    NB_PRINT_IN_MEANTMFILE,
    PRINT_MEANTMDELTA,
    NB_MEANTMFILE_MAX,
    MEANTMFILE_TYPE,
    "MEANTMedium"
  },
  {
    NB_STEP_IN_MMEANTMFILE,
    NB_PRINT_IN_MMEANTMFILE,
    PRINT_MMEANTMDELTA,
    NB_MMEANTMFILE_MAX,
    MMEANTMFILE_TYPE,
    "MeanMeanT"
  },
  {
    NB_STEP_IN_MATDFILE,
    NB_PRINT_IN_MATDFILE,
    PRINT_MATDDELTA,
    NB_MATDFILE_MAX,
    MATDFILE_TYPE,
    "Matd"
  },
  {
    NB_STEP_IN_MEANMATDFILE,
    NB_PRINT_IN_MEANMATDFILE,
    PRINT_MEANMATDDELTA,
    NB_MEANMATDFILE_MAX,
    MEANMATDFILE_TYPE,
    "MeanMatd"
  }
};

HmiTool_t HmiTool[NB_HMI_TOOL] =
{
	{          // SciLab
		100,     // 300 is the limit.
		"sl",
		"//",
    0
	},
	{         // GnuPlot
		100000, // More than 100 000 impossible to whatch particles, everything is red.
		"gnu",
		"#",
    #if (NB_DIM == 3)
      "PlotGnuTmp3D.gnu"
    #else
      "PlotGnuTmp.gnu"
    #endif
	},
	{         // GnuPlot speed radial dispersion
		100000,      // 0 if all the particles will be plotted.
		"gnu",
		"#",
    "PlotGnuTmpDisp.gnu"
	},
	{         // GnuPlot speed tangential dispersion
		100000,
		"gnu",
		"#",
    "PlotGnuTmpDisp.gnu"
	},
	{         // GnuPlot speed sigma radial
		1,      // That s a curve !
		"gnu",
		"#",
    "PlotGnuTmpSigma.gnu"
	},
	{         // GnuPlot speed sigma tangential
		1,      // That s a curve !
		"gnu",
		"#",
    "PlotGnuTmpSigma.gnu"
	},
	{         // GnuPlot speed mean tangential
		1,      // That s a curve !
		"gnu",
		"#",
    "PlotGnuTmpMean.gnu"
	},
	{         // GnuPlot mean of speed mean tangential
		1,      // That s a curve !
		"gnu",
		"#",
    "PlotGnuTmpMean.gnu"
	},
	{         // GnuPlot matter density
		1,      // That s a curve
		"gnu",
		"#",
    "PlotGnuTmpMatd.gnu"
	},
	{         // GnuPlot mean matter density
		1,      // That s a curve
		"gnu",
		"#",
    "PlotGnuTmpMatd.gnu"
	}
};
