/*   INITIALISATION OF THE GROUP OF PARTICLES (initialisation of the "galaxy").
*/
/**************************************** INITIALISED GALAXY STRUCTURE ***************************************/
/* ---------------------------------------------- Init types ----------------------------------------------- */
#define INIT_LINE            0						// Line configuration.
#define INIT_SQUARE          1						// Square configuration.
#define INIT_DISC_CST        2						// Disk configuration with constant matter density.
#define INIT_DISC_EXP        3						// Disk configuration with exponential matter density law.
#define INIT_DISC_CST_EQ     4						// Disk conf constant matter density and in equilibrium.
#define INIT_SOLSYST         5                       // Solar system.
#define INIT_SOLSYST_3       6                       // Solar system for grid3 calculation.
#define INIT_DISC_HOHL       7                       // HOHL (1971) type init.
#define INIT_3D_CUBE         8                       // Cube constant matter density.
#define INIT_3D_SOLSYST      9                       // Solar system in a chosen plan in a cube.
#define INIT_3D_SOLSYST_OBL  10                      // Solar system in a chosen plan in a cube.
#define INIT_BULLET_CLUSTER  11                      // Init of a simulation of Bullet cluster

/* ------------------------------------------- Choosing Init type ------------------------------------------ */
#if MODE_DEBUG == DEBUG_SOLSYST                     // If test of solar system.
  #if NB_DIM == 3                                   // NB_DIM == 3
    #define INIT_TYPE        INIT_3D_SOLSYST_OBL 	  // Init sol syst here is forced.
  #else                                             // NB_DIM == 2
    #define INIT_TYPE        INIT_SOLSYST_3	   	    // Init sol syst here is forced.
    #undef OLDVERSION 
  #endif
#elif STET_MODE != STET_BULLET                                                            
  #if NB_DIM == 3
    #define INIT_TYPE        INIT_3D_CUBE     
  #else                                             // NB_DIM = 2
    #define INIT_TYPE        INIT_DISC_HOHL    //INIT_DISC_HOHL	   	    // Choose one in the above list.
  #endif
#else                                          // STET_MODE == STET_BULLET
  #define INIT_TYPE          INIT_BULLET_CLUSTER    
#endif
/* -------------------------------------- Solar system configuration --------------------------------------- */
#define NBPART_SYSTSOL    10          // FOR TESTS   
/* ----------------------------------------- Line configuration ------------------------------------------ */
#define NBPART_LINE		    3
/* ----------------------------------------- Square configuration ------------------------------------------ */
#define NBPART2			      10  
#define NBPART_SQUARE	    (NBPART2 * NBPART2)
/* ----------------------------------------- Cube configuration ------------------------------------------ */
#define DT_CUBE           (DAY)
#define NBPART3			      60  
#define NBPART_CUBE	      (NBPART3 * NBPART3 * NBPART3)
#define PART_DIST_CUBE    EARTH_SUN_DIST
#define PART_MASS_CUBE    MASS_SUN
#define LG_GRID_CUBE      (NBPART3 * PART_DIST_CUBE)
  //#define SIGMA_POS_CUBE    (PART_DIST_CUBE / 8)
#define SIGMA_POS_CUBE    (PART_DIST_CUBE / 16)
  //#define SIGMA_V0_CUBE     (PART_DIST_CUBE / YEAR)
#define SIGMA_V0_CUBE     0
/* ------------------------------------------ Disk configuration ------------------------------------------- */
#define MASS_MILKYWAY_CORR   (1.0*MASS_MILKYWAY)         // Not used for HOHL like init.
  //#define IR_MAX               218    // HOHL : 10 : 253; 220: 102895 part; 218 : 101429 particles.
#define IR_MAX               2200 //2200    // HOHL : 10 : 253; 220: 102895 part; 218 : 101429 particles. 500:527576. 
                                     // 700:1032176. 900:1704496. 1000:2104356. 2000:>5000000. 1500:4727442.
                                     // 2000:8397726. 2100:9256181. 2200:10159669.
  //#define IR_MAX              120  // EQ : 3: 37 part. 10: 300 p, 50: 7987, 100: 31581, 120: 45556, 200: 126192.
#define DR_MIN               (0.75 * PART_DIST)         // dr is minimised for avoid too low dt: 0.039. With this 0.24.                 
/* ----------------------------------- Disk exponential configuration ----------------------------------- */

/* ----------------------------------- HOHL LIKE disk configurations ------------------------------------ */
/* ---------------------------------------- Real HOHL configuration ------------------------------------- */
#define R_DISK_HOHL          (8 * KILPARSEC)                          // Refer "III AXISYMMETRIC EVOLUTION".
#define DIAM_DISK_HOHL       (2*R_DISK_HOHL)
#define TOTAL_MASS_HOHL      (100000 * 0.84E6 * MASS_SUN)    // Report legend of Fig 9 of 1971 HOHL article.
#define MU0_HOHL             (3 * TOTAL_MASS_HOHL / (2 * PI * R_DISK_HOHL * R_DISK_HOHL))      // Calcul simple integral.
#define OMEGACORR_FORDISP_HOHL  0.809     // Corr multiplictive term for compensat velocity dispersion. HOHL.
#define OMEGA0_HOHL          (OMEGACORR_FORDISP_HOHL * PI * sqrt(G_CONST * MU0_HOHL / (2 * R_DISK_HOHL)))  // HOHL.
#define WIDTH_GALAXY_HOHL    (WIDTH_MILKYWAY * R_DISK_HOHL/R_MILKYWAY)        // Supposed proportional.

/* ---------------------------------------- Milky-way configuration ------------------------------------- */
#define MASS_MILKYWAY	       (5.8E11 * MASS_SUN)   // Goes with IR_MAX = 8, fitted.
#define R_MILKYWAY           (17 * KILPARSEC)
#define DIAM_MILKYWAY        (2 * R_MILKYWAY)
#define MU0_MILKYWAY         (3 * MASS_MILKYWAY / (2 * PI * R_MILKYWAY * R_MILKYWAY))      // Calcul simple integral.
#define OMEGACORR_FORDISP_MILKYWAY  0.809     // Corr multiplictive term for compensat velocity dispersion. HOHL.
#define OMEGA0_MILKYWAY      (OMEGACORR_FORDISP_HOHL * PI * sqrt(G_CONST * MU0_MILKYWAY / (2 * R_MILKYWAY)))  // HOHL.
#define WIDTH_GALAXY_MILKYWAY WIDTH_MILKYWAY

#define RING_FACTOR 1   // simu 5     5:OK

/* ----------------------------------- HOHL like USED disk configuration -------------------------------- */
#if GRAVIT_MODEL == STET && STET_MODE == STET_COMPLEX 
    //  #define MU0                  MU0_MILKYWAY              
    //  #define OMEGA0               OMEGA0_MILKYWAY 
    //  #define SPEED_GAUSS_SIGMA    0.341                  // Sigma of Gauss disporsion of initial velocities. HOHL.                 
    //  #define TOTAL_MASS           MASS_MILKYWAY 
    //  #define GALACTIC_WIDTH       WIDTH_GALAXY_MILKYWAY
  #define MU0                  MU0_HOHL              
//  #define OMEGA0               OMEGA0_HOHL 
  #define OMEGA0               (OMEGA0_HOHL/RING_FACTOR)   // simu 5 
  #define SPEED_GAUSS_SIGMA    0.341                  // Sigma of Gauss disporsion of initial velocities. HOHL.                 
//  #define TOTAL_MASS           TOTAL_MASS_HOHL 
  #define TOTAL_MASS           (TOTAL_MASS_HOHL/(RING_FACTOR*RING_FACTOR))    // simu 5 
  #define TOTAL_MASS0          (TOTAL_MASS_HOHL)                              // simu 5 but for all. 
  #define GALACTIC_WIDTH       WIDTH_GALAXY_HOHL
#else                   // NEWTON OR STET SIMPLE : Real HOHL init. 
  #define MU0                  MU0_HOHL              
//  #define OMEGA0               OMEGA0_HOHL 
  #define OMEGA0               (OMEGA0_HOHL/RING_FACTOR)   // simu 5 
  #define SPEED_GAUSS_SIGMA    0.341                  // Sigma of Gauss disporsion of initial velocities. HOHL.                 
  #define TOTAL_MASS0           TOTAL_MASS_HOHL 
//  #define TOTAL_MASS           TOTAL_MASS_HOHL 
  #define TOTAL_MASS           (TOTAL_MASS_HOHL/(RING_FACTOR*RING_FACTOR))    // simu 5 
  #define GALACTIC_WIDTH       WIDTH_GALAXY_HOHL
#endif 

/************************************ OTHER PARTICLES S CHARACTERISTICS ************************************/
#define PART_MASS		         MASS_SUN                      // Default value only. Not used for HOHL init.	
#define CORR			           0.999                         // For avoiding easily a null denominator.
  //#define V0				V_STARS_MILKYWAY              // 29806 is Earth speed in m/s. Here 250 km/s
#define V0				V_EARTH
#if (NB_DIM == 3)
  #define V0_3D			V_EARTH     // TO BE FITTED ?
#endif
#if INIT_TYPE == INIT_DISC_CST_EQ   ||  INIT_TYPE == INIT_DISC_HOHL
  #define PART_DIST			(R_GALAXY / IR_MAX)
#else
  #define PART_DIST			EARTH_SUN_DIST             // Earth to Sun distance in m. Not enough for a galaxy.
#endif

/* ---------------------------------------- Particle's group ray ------------------------------------------ */
#if MODE_DEBUG == DEBUG_SOLSYST
  #define R_GALAXY			        EARTH_SUN_DIST
#else
  #if INIT_TYPE == INIT_DISC_HOHL
    #if GRAVIT_MODEL == STET && STET_MODE == STET_COMPLEX            // HOHL like init but Milky-way.
               //      #define R_GALAXY			      R_MILKYWAY
      #define R_GALAXY			    R_DISK_HOHL
    #else                                                            // NEWTON AND STET SIMPLE : Real HOHL init. 
      #define R_GALAXY			    R_DISK_HOHL
    #endif 
  #elif INIT_TYPE == INIT_BULLET_CLUSTER
    #define R_GALAXY			      (BULLET_LG_GRID/2)
  #else
  #endif
#endif

extern double omega0;   // Here for debug.

/***************************************** MACROS and FUNCTIONS *****************************************/
extern void Init(int InitType);
extern void SunFixSolSyst();
extern double GetSpeedInit(int PartNum);
extern void ReInitPartAccel(long int StepNum);
extern void Cooling();
extern double NormalGauss();