/*************************************************************************************************************/

#include <stdlib.h> 
#include <string.h> 
#include <stdio.h>
#include <math.h>
#include "common.h"
#include "init.h"
#include "particles.h"
#include "hmi.h"
#include "stat.h"
#include "stet.h"
#include "potential.h"
#include "bulletcluster.h"

// Pour tester fft lib code :
//extern double Pi;
//typedef double_complex cplx;
//extern void fft(cplx buf[],int n);

long int StepNum;            // StepNum. Here for debug and tracking precision and time.
int totaltime;               // For HMI only.

void Simulate(long int FirstStepNum, long int LastStepNum)
{
	for(StepNum = FirstStepNum; StepNum <= LastStepNum ; StepNum++)
	{
    /******* Plotting ******************************************************/
		if(StepNum % PRINT_ECHO == 0) PrintSimulInfoInFile(stdout,0);
		PlotOpenCloseFileIfNeeded(StepNum);
		PlotPartsOneStep(StepNum);   // Print posold for each particle.

    /******* Specific action if solar system configuration *****************/
		SunFixSolSyst();                // If needed, forcing partic simulating 
                                 // sun to be fixed on same location.
    /******* Calculate new gravitational field *******************/
    #if PERIODIC_CALCUL == 0
      RecenterGrid();
    #endif
    DensityCalculate();           // Counting nb of part in each cell.
    ReInitStet();

		GravitPotentialCalculate();   // Calculate gravitational potential.

    /******* Calculate new particles's position *****************************/
		ProcessPosit();              // Calculation of positioning the particles.
		total_time = CalculTime(total_time);   // Add dt to time. For plotting.

    /************* Statistics **********************************************/
    #if PERIODIC_CALCUL == 0
      StatCalc();                 // Report speed dispersion for plotting.
    #endif

    #if MODE_DEBUG == DEBUG_NORM  &&  GRAVIT_MODEL == STET
      TestStet();   
  	  //TestStat();     // When testing
    #endif
	}
}

int main(int argc,char *argv[])
{
  #if INIT_TYPE == INIT_BULLET_CLUSTER 
    BulletCalc();
  #else

	int NStep;

	fprintf(stdout," ----------------- DEBUT -------------- \n");

  /************** Initialisation of modules *********************************/
	InitParticles();          // Init of particles.c module.
  InitPotential();          // Init of potential.c module.
	Init(INIT_TYPE);          // Init of init.c module (whole simulation init).
  InitStet();               // Init of stet.h module. Executed after Init().
	InitHmi();                // Init of hmi.c module.

  /**************** Simulating and cooling regularely **********************/
  for(NStep=0 ; NStep<NB_STEP_FIRSTPHASE ; NStep+=INCR_SIMU)
  {
    Simulate(NStep, NStep+INCR_SIMU-1);
    Cooling();
  }
  for(NStep=NB_STEP_FIRSTPHASE ; NStep<NB_STEP_SECONDPHASE ; NStep+=INCR_SIMU2)
  {
    Simulate(NStep, NStep+INCR_SIMU2-1);
    Cooling();
  }
  Simulate(NB_STEP_SECONDPHASE, NB_STEP);

  /**************** Simulation ending if ever needed ***********************/
	PlotEndFile();        
	printf(" ----------------- FIN -------------- \n");
  //scanf("%d",&tmp);

  #endif       // Bullet cluster case.
	return 0;
}

/********************* Error function *************************/
void Error(int ErrNum)
{
	int tmp;
	printf("ERROR, ErrNum=%d\n", ErrNum);
	scanf("%d",&tmp);
}

double CalculTime(double total_time)
{
	return(total_time + Gravit_dt);
}
