/******************************************* PARTICLES and DYNAMIC *******************************************/
/* ------------------------------------------- Array of particles ------------------------------------------ */
#define NBMAX_PART        12000000					// max 22000000  10 000 000 ok 15 nok

/************************************************** TIME ***************************************************/
#if MODE_DEBUG == DEBUG_SOLSYST
  #define REVOL_TIME      YEAR                              // Real galactic revolution time.
  #define DT              (10*DAY)                          // Ca marche avec 10 particules et 256 cellules.
#elif MODE_DEBUG == DEBUG_NORM
  #define REVOL_TIME      REVOL_TIME_MILKYWAY               // Real galactic revolution time.
  #define NBSTEP_REVOL    200                               // 200 for HOHL.
  #define DT              (REVOL_TIME/NBSTEP_REVOL)         // Required slice (step) time for those values above.
#elif MODE_DEBUG == RELEASE
  #define CALC_TIME       (7*MIN)                           // Compute time for one galaxy revolution.
  #define REVOL_TIME      REVOL_TIME_MILKYWAY                         // Real galactic revolution time.
  #define SIMUL_TIME      REVOL_TIME                        // Total real time for this simulation.
  #define STEP_CALC_TIME  0.001                             // In seconds. 
  #define NBSTEP          (CALC_TIME/STEP_CALC_TIME)        // Nb of step for one galaxy revolution.
  #define PRECIS_RATIO    0.0003                            // Used for the calculation of Dt in the code.
  #define NBSTEP_REVOL    200                               // 200 for HOHL.
//  #define DT              (REVOL_TIME/NBSTEP_REVOL)         // Required slice (step) time for those values above.
#define DT              (RING_FACTOR * REVOL_TIME/NBSTEP_REVOL)         // simu 5 and for all after that.
    //  #define DT              ((REVOL_TIME/NBSTEP_REVOL)/9.27 )         // Required slice (step) time for those values above.
    //  #define DT             (SIMUL_TIME/NBSTEP)            // Required slice (step) time for those values above.
#else
#endif

/******************************************* PARTICLE S STRUCTURE ****************************************/
typedef struct{
	double pos[3];
	double posold[3];
	double posnew[3];
	double acc[3];
	int	   disc;                   // Discarded (for plotting) particle if 1, else 0. But allways calculated.
	double nbpart_in_sphere;       // For STET only. Nb of parts in the sphere of ray rmax around this partic.
	double PosNorm;
  #if STET_MODE == STET_COMPLEX
    unsigned int NumCell[3];   
  #endif   // End of <<#if GRAVIT_MODEL == STET>> 
}particle_t;

/********************************************* GRID VARIABLES ********************************/
/* -------------------------------------------- Grid size ---------------------------------- */
#define NBCELLX_GRID              256  //1024  // 256 for HOHL. 32 in 3D good compromise. TOUCH THIS ONE.
#define NBCELLY_GRID              NBCELLX_GRID  //  //(256+128) //64   //              // TOUCH THIS ONE.
#if (NB_DIM == 3)                                                  // TOUCH THIS ONE.
  #define NBCELLZ_GRID            64  //64
#else
  #define NBCELLZ_GRID            1   // Enabling to execute 2D with a 3D code.
#endif
                                      //  A la main:
                                      //  64 : Part Num 100000, acc0 = -2.8467920653523880e-012
                                      // 256 :                         -3,92 E-12.
                                      // 512 :                         -4.0096217525494803e-012
#define NBCELL_GRID               (NBCELLX_GRID * NBCELLY_GRID)
#define GRIDSQUARE_RAY            (NBCELLX_GRID/2)          // Ray of grid square. Dont touch.

/* -------------------- Location of the grid from respect to the galaxy ------------------- */
#if  MODE_DEBUG==RELEASE  ||  MODE_DEBUG==DEBUG_NORM    // TEMPORARY
  #if (NB_DIM == 3)                                     // NB_DIM == 3
    #if (INIT_TYPE == INIT_3D_CUBE)     
      #define LG_GRID               LG_GRID_CUBE
    #else
      #define LG_GRID               (12 * DIAM_MILKYWAY)
    #endif
  #else         // NB_DIM == 2
    #if GRAVIT_MODEL==NEWTON
      #if INIT_TYPE == INIT_BULLET_CLUSTER 
        #define LG_GRID             BULLET_LG_GRID
      #else
        #define LG_GRID             (12 * DIAM_DISK_HOHL)
      #endif
    #elif GRAVIT_MODEL==STET
      #if INIT_TYPE == INIT_BULLET_CLUSTER 
        #define LG_GRID             BULLET_LG_GRID
      #else                                                 // HOHL dim 2 :
        #define LG_GRID             (24 * DIAM_DISK_HOHL)   // Ray = 24 * 8 = 192 kpc Interessant mais vitesses croissent.
//        #define LG_GRID             (48 * DIAM_DISK_HOHL)   // 
      #endif
    #endif
  #endif
#elif  MODE_DEBUG==DEBUG_SOLSYST
    #if GRAVIT_MODEL==NEWTON
      #define LG_GRID                   (1.2 * 2 * R_GALAXY) // 1.2 is more sure. Testing different modes.  
    #elif GRAVIT_MODEL==STET
        //#define LG_GRID                   (0.5 * 2 * R_GALAXY) // Test of PositOut.
        //      #define LG_GRID                   (2*1.2 * 2 * R_GALAXY)   // DEBUG 
      #define LG_GRID                   (1.2 * 2 * R_GALAXY)   // Normal value for PositIn.
    #endif                                                  //  exactly, along with NbCellX = 64.
#else
#endif

#define LGX_CELL                (LG_GRID / ((double) NBCELLX_GRID))
#define LGY_CELL                (LG_GRID / ((double) NBCELLY_GRID))
#define LGZ_CELL                (LG_GRID / ((double) NBCELLZ_GRID))
#define GRID_ORIGIN0            (- ((double) LG_GRID)/2.) 
#define GRID_ORIGIN1            (- ((double) LG_GRID)/2.) 
#define GRID_ORIGIN2            (- ((double) LG_GRID)/2.) 
#define MAX_GALACTIC_DISPL      (LG_GRID/20.)

/* ----------------------------------- Return code of function GetCell() -------------------------- */
#define CELL_IN                 0
#define CELL_OUTX               1
#define CELL_OUTY               2
#define CELL_OUTZ               3

/* -------------------------------------------- FFT VARIABLES ------------------------------------- */
/* Those are FFT variables which are used, by the way, even when no FFT calculation is done. That's */
/* for simplification of the code.                                                                  */
#define NBCELLX_FFT            (2*NBCELLX_GRID)                        // Of course for convolution.
#define NBCELLY_FFT            (2*NBCELLY_GRID)                        // Of course for convolution.
#if (NB_DIM == 3)
  #define NBCELLZ_FFT          (2*NBCELLZ_GRID)                        // Of course for convolution.
#else
  #define NBCELLZ_FFT          1
#endif
#define NBCELL_FFT             (NBCELLX_FFT * NBCELLY_FFT * NBCELLZ_FFT)

/******************************************* GLOBAL VARIABLES *******************************************/
/* ----------------------------------------- Engine variables ----------------------------------------- */
#if INIT_TYPE != INIT_BULLET_CLUSTER
  extern particle_t Part[NBMAX_PART];
#else
  extern particle_t Part[1];   // For a correct generation.
#endif
extern double Gravit_dt;
extern unsigned int NbPart;                             // Now calculated dynamically because of complicated init.
extern unsigned int NbPartStetGas;      // Gas particles.
extern unsigned int NbPartOut;
extern double PartMass;
extern double total_time;
extern double PosOrigin[3];
#if NB_DIM == 2 && INIT_TYPE != INIT_BULLET_CLUSTER
  extern unsigned int CellNbPart[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID];
  extern unsigned int CellNbPartStetGas[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID];
           // +2 for allowing the test of x, y, z out of bounds of regular grid.
#elif NB_DIM == 3
  extern unsigned int CellNbPart[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID+2];
  extern unsigned int CellNbPartStetGas[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID+2];
#endif

/***************************************** MACROS and FUNCTIONS *****************************************/
#if (NB_DIM == 3)
  #define  VectorCopy(a, b) b[0] = a[0]; b[1] = a[1]; b[2] = a[2]; 
  #define  VectorZero(a) a[0] = a[1] = a[2] = 0;
  #define  VectorAdd(a, b, c) c[0] = a[0] + b[0]; c[1] = a[1] + b[1]; c[2] = a[2] + b[2];
  #define  VectorSub(a, b, c) c[0] = a[0] - b[0]; c[1] = a[1] - b[1]; c[2] = a[2] - b[2];
  #define  VectorDiv(a, b, c) c[0] = a[0]/b; c[1] = a[1]/b; c[2] = a[2]/b;
  #define  VectorMul(a, b, c) c[0] = a[0]*b; c[1] = a[1]*b; c[2] = a[2]*b;
  #define  VectorNorm(a, p_b) *p_b = sqrt(a[0]*a[0] + a[1]*a[1] + a[2]*a[2]);
#elif (NB_DIM == 2)
  #define  VectorCopy(a, b) b[0] = a[0]; b[1] = a[1];
  #define  VectorCopy3D(a, b) b[0] = a[0]; b[1] = a[1]; b[2] = a[2]; 
  #define  VectorZero(a) a[0] = a[1] = 0;
  #define  VectorAdd(a, b, c) c[0] = a[0] + b[0]; c[1] = a[1] + b[1];
  #define  VectorSub(a, b, c) c[0] = a[0] - b[0]; c[1] = a[1] - b[1];
  #define  VectorDiv(a, b, c) c[0] = a[0]/b; c[1] = a[1]/b;
  #define  VectorMul(a, b, c) c[0] = a[0]*b; c[1] = a[1]*b;
  #define  VectorNorm(a, p_b) *p_b = sqrt(a[0]*a[0] + a[1]*a[1]);
#endif
extern void InitParticles();
extern void ProcessPosit();
extern void DensityCalculate();
extern void RecenterGrid();
extern unsigned char GetCell(double pos[], unsigned int NumCell[]);
extern void GravitFieldCalculate();    // This function is only used at init time for some cases.
