/*   Here the simulated universe is understood as periodic.
*/

#include <stdlib.h> 
#include <string.h> 
#include <stdio.h>
#include <math.h>
#include "common.h"
#include "period.h"

#if PERIODIC_CALCUL == 1

// Periodic mechanism. As an example, this funtion below:
// INPUT :   x is the signed integer which points the cell.
//           period is the number of cells in the grid.
// OUTPUT :  x is the UNsigned integer which points the cell INSIDE the grid
//           in a periodic representation.
// Therefore :
//           x >= 0 : rest of arithmetic euclidean division.
//           x < 0 :  rest of arithmetic euclidean division BUT
//                    "counted from the left".
unsigned int GetModuloPeriodic(int x, unsigned int period)    // TESTED
{
	if(x >= 0)
	{
		x = x % period;                   // That s simple !
	}
  else                                // x < 0
	{
    unsigned r = (-x) % period;       // Euclidean division with positive numbers. 
    if(r != 0) x = period - r;        // And in this case 0 < x < period 
    else x = 0;
	}
	return(x);
}

double GetDoubleModuloPeriodic(double x)     // TESTED
{
	if(x >= 0)
	{
		x = x - ((int) (x / LG_GRID)) * LG_GRID;                   // That s simple !
	}
  else                                // x < 0
	{
	  x = x - ((int)(x / LG_GRID)) * LG_GRID;
	  x = LG_GRID + x;
  }
	return(x);
}

/* --------------------- Get the cell containing a given particle -------------------- */
unsigned char GetCellPeriod(double pos[], unsigned int NumCell[])         // TESTED
{
  double Offset;
  int NumC;

  Offset = pos[0];    
  NumC = (int) (Offset / LGX_CELL);
  NumC = (Offset >= 0) ? NumC : NumC-1;
  NumCell[0] = GetModuloPeriodic(NumC, NBCELLX_GRID);

  Offset = pos[1];    
  NumC = (int) (Offset / LGY_CELL);
  NumC = (Offset >= 0) ? NumC : NumC-1;
  NumCell[1] = GetModuloPeriodic(NumC, NBCELLY_GRID);

  #if (NB_DIM == 3)
    Offset = pos[2];    
    NumC = (int) (Offset / LGZ_CELL);
    NumC = (Offset >= 0) ? NumC : NumC-1;
    NumCell[2] = GetModuloPeriodic(NumC, NBCELLZ_GRID);
  #else
    NumCell[2] = 0;
  #endif

  return(CELL_IN);          // Never OUT for this version.
}

#else  // NON PERIODIC CASE. This allows to exec same code for Per AND non Period.
  #define GetModuloPeriodic(x, period)    (x) 
#endif
