/*   Calculation of potential
*/

#include <stdlib.h> 
#include <string.h> 
#include <stdio.h>
#include <math.h>
#include "common.h"
#include "hmi.h"
#include "init.h"
#include "particles.h"
#include "fft.h"
#include "stet.h"
#include "period.h"
#include "bulletcluster.h"
#include "potential.h"

/* ************************* Engine variables ******************************* */
/* -------------------- Potential and particles domains --------------------- */
double Potential[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID]; 
unsigned char PotentialDisk[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
#if NB_DIM == 2
  unsigned char GridDisk[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID];
           // +2 for allowing the test of x, y, z out of bounds of regular grid.
#elif NB_DIM == 3
  unsigned char GridDisk[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID+2];
#else
#endif

/* -------------------- Optimizing inverse distance array ------------------- */
double InvDist[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];

/* ------------------------- Grid3 optimisation arrays ---------------------- */
#if MODE_CALCUL_POTENTIAL == MODE_GRID3 
  unsigned char Grid3Disk[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
  unsigned char GridOptimDisk[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
  unsigned char GridOptimSquare[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
#endif

/* ----------------------- Local functions prototypes ---------------------- */
void InitInvDist();

/* ---------------------------- Module INIT -------------------------------- */
void InitPotential()                          // Mainly optimisation disks INIT 
{
  unsigned int jx, jy, jz;

  if(MODE_CALCUL_POTENTIAL == MODE_FFT) InitFFT();
                 // But init GridDisk below in any potetial mode.

  memset(Potential, 0, sizeof(Potential));
  InitInvDist();
  memset(PotentialDisk, 0, sizeof(PotentialDisk));
  memset(GridDisk, 0, sizeof(GridDisk));
  #if MODE_CALCUL_POTENTIAL == MODE_GRID3
    memset(Grid3Disk, 0, sizeof(Grid3Disk));
    memset(GridOptimDisk, 0, sizeof(GridOptimDisk));
    memset(GridOptimSquare, 0, sizeof(GridOptimSquare));
  #endif

  for(jy=0 ; jy<NBCELLY_GRID ; jy++)
  {
    for(jx=0 ; jx<NBCELLX_GRID ; jx++)
    {
      for(jz=0 ; jz<NBCELLZ_GRID ; jz++)
      {
        long int x = jx - ((long int) GRIDSQUARE_RAY);
        long int y = jy - ((long int) GRIDSQUARE_RAY);
        #if (NB_DIM == 3)
          long int z = jz - GRIDSQUARE_RAY;
        #else
          long int z = 0;
        #endif
			  if(x*x + y*y + z*z  <=  GRIDDISK_RAY * GRIDDISK_RAY)
			  {          
          GridDisk[jx][jy][jz] = 1;             // This is inside the disk.
        }                                       // Avoid square corners.
        else ; 

        #if PERIODIC_CALCUL == 0
			    if(x*x + y*y + z*z  <=  POTDISK_RAY * POTDISK_RAY)
			    {          
            PotentialDisk[jx][jy][jz] = 1;      // This is inside the disk.
          }                                     // Dont calculate if not needed.
          else ; 
        #elif PERIODIC_CALCUL == 1      // Here potential calculated everywhere.
          PotentialDisk[jx][jy][jz] = 1;
        #endif 

        #if MODE_CALCUL_POTENTIAL == MODE_GRID3
			    if(x*x + y*y + z*z  <=  GRID3DISK_RAY * GRID3DISK_RAY)
			    {          
            Grid3Disk[jx][jy][jz] = 1;            // This is inside the disk.
          }                                    
          else ; 
          if(jx*jx + jy*jy + jz*jz  <=  GRIDOPTIMDISK_RAY * GRIDOPTIMDISK_RAY)
          {   
            GridOptimDisk[jx][jy][jz] = 1;       // This is inside the disk.
          }
          else ;                                 // Avoid square corners.
          if(jx <= GRIDOPTIMSQUARE_RAY  &&  jy <= GRIDOPTIMSQUARE_RAY  &&  jz <= GRIDOPTIMSQUARE_RAY)
          {   
            GridOptimSquare[jx][jy][jz] = 1;        // This is inside the square.
          }
          else ;                                
        #endif
      }
    }
  }
}

/* --------------------- InverseDistance array INIT -------------------- */
void InitInvDist()
{
  unsigned int jx, jy, jz;
  long double dx0, dx1, dx2, dx, distance;
  
  memset(InvDist, 0, sizeof(InvDist));

  /* --- Calculate inverse distance in small grid ----------- */
  for(jy=0 ; jy<=NBCELLY_GRID ; jy++)
  {
    for(jx=0 ; jx<=NBCELLX_GRID ; jx++)
    {
      for(jz=0 ; jz<=NBCELLZ_GRID ; jz++)
      {
        dx0 = jx * LGX_CELL;
        dx1 = jy * LGY_CELL;
        dx2 = jz * LGZ_CELL;
        dx = dx0*dx0 + dx1*dx1 + dx2*dx2;
        distance = sqrt(dx);
        if(jx!=0 || jy!=0 || jz!=0)              // Avoid division by zero.
        {
          InvDist[jx][jy][jz] = G_CONST / distance;  // Small grid (first quarter))
        }else ;
      }
    }
  }
  /* --- First corner of small and also large grid ---- */
  distance = LGX_CELL;
  InvDist[0][0][0] = G_CONST / distance;
}

void AddPotential_CLOSELargeCell(unsigned int ix, unsigned int iy, unsigned int iz, int x, int y, int z, unsigned int *pNbPartInStet)
{
  double CellMass;
	int sx, sy, sz;
  unsigned int ax, ay, az;

  /************* Potential due to THIS large cell which is very close *****/
  for(sx = x - GRIDOPTIM_LARGECELL_RAY ; sx <= x + GRIDOPTIM_LARGECELL_RAY ; sx++)
  {
    for(sy = y - GRIDOPTIM_LARGECELL_RAY ; sy <= y + GRIDOPTIM_LARGECELL_RAY ; sy++)
    {
      #if (NB_DIM == 3)
        for(sz = z - GRIDOPTIM_LARGECELL_RAY ; sz <= z + GRIDOPTIM_LARGECELL_RAY ; sz++)
        {
      #else
        sz = 0;
      #endif
          if(IsValidCell(sx, sy, sz))    
          {                         // If this cell is inside the disk.
            CellMass = CellNbPart[sx][sy][sz] * PartMass;
            ax = GridDistance(sx-ix); ay = GridDistance(sy-iy); az = GridDistance(sz-iz);
            Potential[ix][iy][iz] += CellMass * InvDist[ax][ay][az];
            #if GRAVIT_MODEL == STET        // For bullet here CellNbPartStetGas is used.
              if(InsideStetDisk(ix, iy, iz, ax, ay, az))
              {                    // Only for cell inside Stet disk.
                *pNbPartInStet += CellNbPartStetGas[sx][sy][sz]; 
              }                    // At the end nb of particles in Stet disk is memorized.
              else continue;       // Avoid : outside Stet disk.  
            #endif
          }
          else ;                   // Avoid if out of GridDisk.
      #if (NB_DIM == 3)
        }      // end for sz
      #endif
    }          // end for sy
  }            // end for sx
}           

unsigned long int GetNbP_FARLargeCell(int x, int y, int z)
{
  unsigned long int NbP;

  /************* Potential due to THIS large cell NOT VERY close *****/
  NbP =   CellNbPart[x-1][y-1][z]        //  Calculate each small cell.
        + CellNbPart[x-1][y]  [z]        // Should use here
        + CellNbPart[x-1][y+1][z]        //  GridDistance
        + CellNbPart[x]  [y-1][z]        // but too slow and
        + CellNbPart[x]  [y]  [z]        // not important
        + CellNbPart[x]  [y+1][z]        // specially when
        + CellNbPart[x+1][y-1][z]        // NBCELLX_GRID is
        + CellNbPart[x+1][y]  [z]        // great.
        + CellNbPart[x+1][y+1][z];
  #if (NB_DIM == 3)
    NbP +=  CellNbPart[x-1][y-1][z-1]
          + CellNbPart[x-1][y]  [z-1]
          + CellNbPart[x-1][y+1][z-1]
          + CellNbPart[x]  [y-1][z-1]
          + CellNbPart[x]  [y]  [z-1]
          + CellNbPart[x]  [y+1][z-1]
          + CellNbPart[x+1][y-1][z-1]
          + CellNbPart[x+1][y]  [z-1]
          + CellNbPart[x+1][y+1][z-1];
    NbP +=  CellNbPart[x-1][y-1][z+1]
          + CellNbPart[x-1][y]  [z+1]
          + CellNbPart[x-1][y+1][z+1]
          + CellNbPart[x]  [y-1][z+1]
          + CellNbPart[x]  [y]  [z+1]
          + CellNbPart[x]  [y+1][z+1]
          + CellNbPart[x+1][y-1][z+1]
          + CellNbPart[x+1][y]  [z+1]
          + CellNbPart[x+1][y+1][z+1];
  #endif
  return(NbP);
}

#if MODE_CALCUL_POTENTIAL == MODE_GRID3

void CalculatePotentialCell3(unsigned int ix, unsigned int iy, unsigned int iz)
{
  double CellMass;
	int x, y, z, MinX, MaxX, MinY, MaxY, MinZ, MaxZ;
  unsigned int ax, ay, az;
  unsigned long int NbP;
  unsigned int NbPartInStet;
  #if GRAVIT_MODEL == STET
    double StetCorr;        // Corrective term added to Newton's acceleration.
    NbPartInStet = 0;
  #endif

	Potential[ix][iy][iz] = 0;                      // And it will be added to this below.

  MinX = ix - (ix/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR;
  MinY = iy - (iy/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR;
  MinZ = iz - (iz/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR;
  MaxX = ix + ((IND_CELL_MAXX-ix)/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR;
  MaxY = iy + ((IND_CELL_MAXY-iy)/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR;
  MaxZ = iz + ((IND_CELL_MAXZ-iz)/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR;

	for(x = MinX ; x <= MaxX ; x+=NBCELL_OPTIM_INCR)
	{
		for(y = MinY ; y <= MaxY ; y+=NBCELL_OPTIM_INCR)
		{                                       // for jx=jy=0, Potential Potential[jx][jy] = 0
      #if (NB_DIM == 3)
        for(z = MinZ ; z <= MaxZ ; z+=NBCELL_OPTIM_INCR)
        { 
      #else
          z = 0;
      #endif
          ax =  GridDistance(x-ix); ay =  GridDistance(y-iy); az =  GridDistance(z-iz);

          if(IsValidBigCell(x, y, z))    // Is x y z is in the grid, with a possible calculation ?
          {                         // If the center of this large cell is inside the disk.
            if(GridOptimSquare[ax][ay][az])    // If this is a very close large square cell.
            {                                             // Large cell close to us.
              /* ------------- Large cell CLOSE : NO optimisation --------------------------- */
              AddPotential_CLOSELargeCell(ix, iy, iz, x, y, z, &NbPartInStet);
                               // Counting of particles in STET disk is done in this function.
            }
            else               // else of: if(GridOptimSquare[ax][ay][az])  
            {                                   // Not VERY CLOSE:
              if(GridOptimDisk[ax][ay][az])     // Far but NOT VERY far:
              {                        // Inside optimizing disk : jx+jy not very strong.

                /* ------------- Large cell FAR : slight optimisation ----------------------- */
                NbP = GetNbP_FARLargeCell(x, y, z);
              }                                          
              else                     // else: GridOptimDisk[ax][ay][az] = 0
              { 

                /* ------------- Large cell VERY FAR : STRONG and dangerous optimisation ---- */
                NbP = CellNbPart[x][y][z] * NBSMALLCELL_IN_LARGECELL; 
                                       // Only centered small cell is read for this large cell. 
              }                                        // end of if GridOptimDisk[ax][ay][az]

              /* ---- Counting number of particles inside STET disk ------------------------- */
              #if GRAVIT_MODEL == STET && STET_MODE != STET_BULLET // Bullet far : NpPStet=0.
                if(InsideStetDisk(ix, iy, iz, ax, ay, az))         // Large cell NOT close to us
                {                    // Only for cell inside Stet disk.
                  NbPartInStet += NbP;
                }                    // At the end nb of all particles in Stet disk is memorized.
                else ;               // Avoid : outside Stet disk.  
              #endif

              /* ---- Updating potential because of this large cell NOT close --------------- */
              CellMass = NbP * PartMass;
              Potential[ix][iy][iz] += CellMass * InvDist[ax][ay][az];
            }                        // end of: if(GridOptimSquare[ax][ay][az])
          }                          // end of  if(IsValidBigCell(x, y, z)
      #if (NB_DIM == 3)
        }     // end for z
      #endif
		}         // end for y
	}           // end for x

  #if GRAVIT_MODEL == STET
    StetCorr = GetStetCorr(NbPartInStet, ix, iy, iz);
    StetCorrMax = max(StetCorr, StetCorrMax);
    StetCorrMin = min(StetCorr, StetCorrMin);
            //    Potential[ix][iy][iz] *= StetCorr;     // Testing correcting accel directly.
    #if STET_MODE == STET_COMPLEX 
      StetCorrMemo[ix][iy][iz] = StetCorr;              // Used in particles.c
    #elif STET_MODE == STET_SIMPLE 
      Potential[ix][iy][iz] *= StetCorr;                // Direct use.
    #endif
       //NbPartInStetDebug[ix][iy][iz] = NbPartInStet;   // DEBUG ONLY !
  #endif
}
#else
  void CalculatePotentialCell3(unsigned int ix, unsigned int iy, unsigned int iz) {}
#endif                       // end of #if MODE_CALCUL_POTENTIAL == MODE_GRID3.

void CalculatePotentialCell(unsigned int ix, unsigned int iy, unsigned int iz)
{
	int x, y, z;
  unsigned int ax, ay, az;
  unsigned long int NbP;
  #if GRAVIT_MODEL == STET
    double StetCorr;        // Corrective term added to Newton's acceleration.
    unsigned int NbPartInStet = 0;
  #endif

	Potential[ix][iy][iz] = 0;                      // And it will be added to this below.

	for(x = 0 ; x <= IND_CELL_MAXX ; x++)
	{
		for(y = 0 ; y <= IND_CELL_MAXY ; y++)
		{                                       // for jx=jy=0, Potential Potential[jx][jy] = 0
      #if (NB_DIM == 3)
        for(z = 0 ; z <= IND_CELL_MAXZ ; z++)
        { 
      #else
          z = 0;
      #endif
          if(IsValidCell(x, y, z))  // Is x y z is in grid disk, with possible calculation ?
          {                            // If the center of this cell is inside the disk.
            ax =  GridDistance(x-ix); ay =  GridDistance(y-iy); az =  GridDistance(z-iz);
            NbP = CellNbPart[x][y][z]; 
            Potential[ix][iy][iz] += NbP * PartMass * InvDist[ax][ay][az];

            /* ---- Counting number of particles inside STET disk ------------------------- */
            #if GRAVIT_MODEL == STET                   // For this large cell NOT close to us.
              if(InsideStetDisk(ix, iy, iz, ax, ay, az))
              {                    // Only for cell inside Stet disk.
                NbPartInStet += CellNbPartStetGas[x][y][z];
              }                    // At the end nb of all particles in Stet disk is memorized.
              else ;               // Avoid : outside Stet disk.  
            #endif
          }
          else ;                       // Out of the grid disk. 
      #if (NB_DIM == 3)
        }     // end for z
      #endif
		}         // end for y
	}           // end for x

  #if GRAVIT_MODEL == STET
    StetCorr = GetStetCorr(NbPartInStet, ix, iy, iz);
    StetCorrMax = max(StetCorr, StetCorrMax);
    StetCorrMin = min(StetCorr, StetCorrMin);
            //    Potential[ix][iy][iz] *= StetCorr;     // Testing correcting accel directly.
    #if STET_MODE == STET_COMPLEX 
      StetCorrMemo[ix][iy][iz] = StetCorr;              // Used in particles.c
    #elif STET_MODE == STET_SIMPLE 
      Potential[ix][iy][iz] *= StetCorr;                // Direct use.
    #endif
       //NbPartInStetDebug[ix][iy][iz] = NbPartInStet;   // DEBUG ONLY !
  #endif
}

void GravitPotentialCalculateByGrid()
{
  int ix;                   // Signed because of parallel need.

  /* ---------------- Parallelisation on multi-core --------------------- */
  #ifdef _OPENMP          // Noticeably avoid parallelisation with STEP by STEP.
    #pragma omp parallel for private( ix )   // Warning doesnt works with step by step. 
  #endif

  /* ---------------- For each cell in the grid calculate its potential - */
  for(ix=0 ; ix<NBCELLX_GRID ; ix++)
  {
    unsigned int iy, iz;
    for(iy=0 ; iy<NBCELLY_GRID ; iy++)
    {
      for(iz=0 ; iz<NBCELLZ_GRID ; iz++)
      {
        if(PotentialDisk[ix][iy][iz])
        {
          switch(MODE_CALCUL_POTENTIAL)
          {
            case MODE_GRID:          // Calculate the integral convolution step by step.
              CalculatePotentialCell(ix, iy, iz); 
              break;
            case MODE_GRID3:         // Calcul integral convol step by step optimized manner.
              CalculatePotentialCell3(ix, iy, iz);
              break;
          }
        }else ;     // No need, out of disk (positout): continue
      }
    }
  }
}

void GravitPotentialCalculate()
{
  switch(MODE_CALCUL_POTENTIAL)
  {
    case MODE_FFT:          // Calculate the convolution by product in freq space.
      GravitPotentialCalculateByFFT();
      break;
    case MODE_GRID:          // Calculate the integral convolution step by step.
    case MODE_GRID3:         // Calcul integral convol step by step optimized manner.
      GravitPotentialCalculateByGrid();
      break;
  }
}

/* -------------- Testing potential in Unit Test ---------------- */
void TestPotential()
{
  unsigned int ix, iy, iz;

  ix = 0;    
  iy = 0;    
  iz = 0;    
  CalculatePotentialCell(ix, iy, iz);
  CalculatePotentialCell3(ix, iy, iz);
}
