/************************ GRID CALCULATION OPTIMISATION VARIABLES ****************************/
/* ------------------------ CALCULATION MODEL FOR THE POTENTIAL ---------------------------- */
#define MODE_GRID                 0
#define MODE_GRID3                1
#define MODE_FFT                  2
#define MODE_CALCUL_POTENTIAL     MODE_GRID3                                 // TOUCH THIS ONE.

/* --------------------------- GRID3 optimisation constants -------------------------------- */
/* Caution: normally there must be the following inclusion:                                  */
/*     SquareRay included into OptimDisk included into GridDisk.                             */
#define NBCELL_OPTIM_INCR       3                         // Dont touch.
#define GRIDOPTIM_LARGECELL_RAY (NBCELL_OPTIM_INCR/2)     // Normally =1. Dont touch.
#define GRIDDISK_RAY            (GRIDSQUARE_RAY - 1 - 1.433)  // Ray of grid disk. Dont touch.
  // 1.433>=sqrt(2) for giving place for Grid3Disk, and -1 for him having place inside of grid.     
#define POTDISK_RAY             (GRIDDISK_RAY + 1)        // Ray of potential disk. Dont touch.
                            // +1 for calculating potential: accel(x) = (pot(x+1)-pot(x-1))/2.     
#define GRID3DISK_RAY           (GRIDDISK_RAY + 1.433)    //Ray scanning grid3disk. Dont touch.
       // 1.433>=sqrt(2) for avoiding scanning grid3 loop bugg: sqrt(2) = diagonal cell length.     
#if STET_MODE == STET_BULLET 
  #define GRIDOPTIMDISK_RAY       (NBCELLX_GRID/4)   // Dangerous optimisation... TOUCH THIS ONE.
#else 
  #define GRIDOPTIMDISK_RAY       (NBCELLX_GRID/6)   // Dangerous optimisation... TOUCH THIS ONE.
#endif 

/* ---- Square ray in which calculation are NOT optimised ---------------------------------- */
  //  #define GRIDOPTIMSQUARE_RAY     151   // DEBUG ONLY
  //#define GRIDOPTIMSQUARE_RAY       31    // Still optimized but quite precise results. 3k + 1 values only.
  //#define GRIDOPTIMSQUARE_RAY     1002    // DEBUG only.
//#define GRIDOPTIMSQUARE_RAY     22          // 22                           // TOUCH THIS ONE.
//#define GRIDOPTIMSQUARE_RAY     67          // 22                           // TOUCH THIS ONE.
#if STET_MODE == STET_BULLET 
  #define GRIDOPTIMSQUARE_RAY     67          // 
#else 
  #define GRIDOPTIMSQUARE_RAY     22          // 22                           // TOUCH THIS ONE.
#endif 
                                     // Greater values better precision poorer performances.
/* ---- LOOP bounds for loop of optimised calculation -------------------------------------- */
#if (NB_DIM == 3)
  #define NBSMALLCELL_IN_LARGECELL  (NBCELL_OPTIM_INCR * NBCELL_OPTIM_INCR * NBCELL_OPTIM_INCR)
#elif (NB_DIM == 2)
  #define NBSMALLCELL_IN_LARGECELL  (NBCELL_OPTIM_INCR * NBCELL_OPTIM_INCR)
#endif

/************************************ VALIDITY OF CELLS **************************************/
/* ----- For loop on big cells ----- */
#define IND_CELL_MAXX (NBCELLX_GRID - 1)             // For loop max bound.
#define IND_CELL_MAXY (NBCELLY_GRID - 1)             // For loop max bound.
#define IND_CELL_MAXZ (NBCELLZ_GRID - 1)             // For loop max bound.

#if PERIODIC_CALCUL == 0 
              // Coded with macros: avoid any call with pop push, for perfs.
  #define IsValidCell(x, y, z)     ((x<0 || y<0 || z<0) ? 0 : GridDisk[x][y][z]) 
                                    // GridDisk is big enough to tolerate biggest call x, y, z.
  #define IsValidBigCell(x, y, z)  ((x<0 || y<0 || z<0) ? 0 : Grid3Disk[x][y][z])
        // Avoiding indirectly fringe of grid, and avoiding grid3 loop "scanning" bugg.
#elif PERIODIC_CALCUL == 1 
  #define IsValidCell(x, y, z)      1       // Every small cell valid in periodic mode.
  // Not very optimized ! Can be better...
  #define IsValidBigCell(x, y, z)                               \   // Avoid limits of    // CAUTION OF POSSIBLE BUGG HERE OF Acceleration steps !!!
  (                                                             \   // grid because
    (    x==0 || x==NBCELLX_GRID-1 || y==0 || y==NBCELLY_GRID-1 \   // this would 
      || z==0 || z==NBCELLZ_GRID-1                              \   // finally get out
    )                                                           \   // of arrays:
    ?                                                           \   // x+1, x-1 ...
    0 : 1                                                       \
  )
#endif 

/* ************************* Engine variables ******************************* */
/* -------------------- Potential and particles domains --------------------- */
extern double Potential[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID]; 

/* --- For used by bullet cluster only: ---- */
extern double InvDist[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
extern unsigned char Grid3Disk[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
extern unsigned char GridOptimDisk[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
extern unsigned char GridOptimSquare[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
extern unsigned char PotentialDisk[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];

#if NB_DIM == 2
  extern unsigned char GridDisk[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID];
           // +2 for allowing the test of x, y, z out of bounds of regular grid.
#elif NB_DIM == 3
  extern unsigned char GridDisk[NBCELLX_GRID+2][NBCELLY_GRID+2][NBCELLZ_GRID+2];
#endif

/*********************************** MACROS and FUNCTIONS **************************/
extern void InitPotential();                        // Mainly optimisation disks INIT 
extern void CalculatePotentialCell(unsigned int ix, unsigned int iy, unsigned int iz);
extern void CalculatePotentialCell3(unsigned int ix, unsigned int iy, unsigned int iz);
extern void GravitPotentialCalculate();
