/*   HANDLING STATISTICS ON PARTICLES (stars in a "galaxy").
*/

#include <stdlib.h> 
#include <string.h> 
#include <stdio.h>
#include <math.h>
#include "common.h"
#include "init.h"
#include "particles.h"
#include "hmi.h"
#include "bulletcluster.h"
#include "stat.h"

#if INIT_TYPE != INIT_BULLET_CLUSTER

/* --------------------------------- Engine variables ------------------------------- */
particle_stat_t PartStat[NBMAX_PART];
SigmaSpeed_t SigmaRadialSpeed[NB_SLICE];
SigmaSpeed_t SigmaTangentSpeed[NB_SLICE];
double PosOr[3];                                    // Coordinates of galactic centroid.
double SurfMattDens[NB_SLICE];

void StatSpeedCalc(SigmaSpeed_t SigmaSpeed[])
{
  unsigned int i, Ind;

  /* ---------------- Calculating azimuthal mean of radial radial and tangent speed ------- */
  memset(SigmaSpeed, 0, NB_SLICE * sizeof(SigmaSpeed_t));
  for(i = 0; i < NbPart; i++)
  {
    if(MODE_DEBUG != DEBUG_SOLSYST)
    {
      Ind = (unsigned int) (PartStat[i].PosNorm / LG_SLICE);
      if(Ind < NB_SLICE)
      {
        SigmaSpeed[Ind].NbPart++;
        if(SigmaSpeed == SigmaRadialSpeed)
        {
          SigmaSpeed[Ind].Mean += PartStat[i].SpeedRadial;
        }
        else
        {
          SigmaSpeed[Ind].Mean += PartStat[i].SpeedTangent;
        }
      }
      else ;
    }
    else ;
  }
  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      if(SigmaSpeed[Ind].NbPart != 0)
      {
        SigmaSpeed[Ind].Mean /= SigmaSpeed[Ind].NbPart;      // Mean value.
      }
      else
      {
        SigmaSpeed[Ind].Mean = 0;                            // No particles....
      }
    }
  }
  else ;  

  /* ---------------- Calculating sigma of azimuthal mean value of radial speed ------- */
  for(i = 0; i < NbPart; i++)
  {
    if(MODE_DEBUG != DEBUG_SOLSYST)
    {
      Ind = (unsigned int) (PartStat[i].PosNorm / LG_SLICE);
      if(Ind < NB_SLICE)
      {
        double RelSpeed; 
        if(SigmaSpeed == SigmaRadialSpeed)
        {
          RelSpeed = PartStat[i].SpeedRadial - SigmaSpeed[Ind].Mean;
        }
        else
        {
          RelSpeed = PartStat[i].SpeedTangent - SigmaSpeed[Ind].Mean;
        }
        SigmaSpeed[Ind].S2 += RelSpeed * RelSpeed;
      }
      else ;    // Just discard if out of grid.
    }
    else ;
  }
  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      if(SigmaSpeed[Ind].NbPart == 0) SigmaSpeed[Ind].S2 = 0;
      else SigmaSpeed[Ind].S2 /= SigmaSpeed[Ind].NbPart;       // Mean value.
      SigmaSpeed[Ind].S2 = sqrt(SigmaSpeed[Ind].S2);      // Variance.
    }
  }
  else ;  
}

void StatCalc()
{
  unsigned int i;
  double PosNorm, Speed[3], Pos[3], RadialVector[3];
    
  /* ---------------- Calculating barycentric coordinates --------------------------- */
  VectorZero(PosOr);
  for(i = 0; i < NbPart; i++)
  {
    VectorAdd(Part[i].pos, PosOr, PosOr);                 // PosOr += pos
  }
  VectorDiv(PosOr, NbPart, PosOr);                        // PosOr = barycentric of Particles posit.

  
  /* ---------------- Memorizing radial and tangential speed for further plot --------- */
  for(i = 0; i < NbPart; i++)
  {
    VectorSub(Part[i].pos, PosOr, Pos);                   // Pos = pos(i) - origin.
    VectorNorm(Pos, &PosNorm);                            // PosNorm = ||Pos||
    if(MODE_DEBUG != DEBUG_SOLSYST)
    {
      if(PosNorm == 0) Error(110);                        // Hardly probable.
      else
      {
        VectorDiv(Pos, PosNorm, RadialVector);            // RadialVector = pos/||pos||
      }
    }
    else ; 
    VectorSub(Part[i].pos, Part[i].posold, Speed);        // Speed = pos - posold.
    VectorDiv(Speed, Gravit_dt, Speed);                   // Speed = (pos - posold) / dt.
    PartStat[i].SpeedRadial  = Speed[0] * RadialVector[0] + Speed[1] * RadialVector[1];   // Scalar product v.ur
    PartStat[i].SpeedTangent = - RadialVector[0]*Speed[1] + RadialVector[1]*Speed[0];     // - Vectorial product ur^v.
    PartStat[i].PosNorm = PosNorm;
  }                                                       // End of for loop over the i particles.

  /* ---------------- Calculating speed stats ---------------------------------------- */
  StatSpeedCalc(SigmaRadialSpeed);
  StatSpeedCalc(SigmaTangentSpeed);
}

void PlotRadSigmaFile(FILE *pFile)
{
  unsigned int Ind;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      double RadialDist = LG_SLICE + LG_SLICE * Ind;
      RadialDist /= PLOT_DIST_UNIT;
      fprintf(pFile, "%f %f\n", RadialDist, SigmaRadialSpeed[Ind].S2 );
    }
  }
  else ;    
}

void PlotTanSigmaFile(FILE *pFile)
{
  unsigned int Ind;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      double RadialDist = LG_SLICE + LG_SLICE * Ind;
      RadialDist /= PLOT_DIST_UNIT;
      fprintf(pFile, "%f %f\n", RadialDist, SigmaTangentSpeed[Ind].S2 );
    }
  }
  else ;    
}

void PlotTanMeanFile(FILE *pFile)
{
  unsigned int Ind;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      double RadialDist = LG_SLICE + LG_SLICE * Ind;
      RadialDist /= PLOT_DIST_UNIT;
      fprintf(pFile, "%f %f\n", RadialDist, SigmaTangentSpeed[Ind].Mean );
    }
  }
  else ;    
}

void PlotSurfacicMatterDensityFile(FILE *pFile)
{
  unsigned int Ind;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      double SliceSurface, SurfMattDensity, RadialDist;
      RadialDist = LG_SLICE + LG_SLICE * Ind;
      SliceSurface = 2 * PI * RadialDist * LG_SLICE - PI * LG_SLICE*LG_SLICE;   // Pi (R2 - R1)
      SurfMattDensity = SigmaTangentSpeed[Ind].NbPart * PartMass / SliceSurface;
      SurfMattDens[Ind] = SurfMattDensity;
      RadialDist /= PLOT_DIST_UNIT;
      fprintf(pFile, "%f %f\n", RadialDist, SurfMattDensity );
    }
  }
  else ;    
}

void PlotMeanSurfacicMatterDensityFile(FILE *pFile)
{
  int Ind, NbInd, i;
  double SurfMattDensity, RadialDist;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      SurfMattDensity = 0;
      NbInd = 0;
      for(i = -R_MEANSURF; i <= R_MEANSURF ; i++)
      {
        if(Ind+i >= 0)
        {
          SurfMattDensity += SurfMattDens[Ind+i];
          NbInd++;
        }
        else ;
      }
      if(NbInd == 0) Error(2000);
      else SurfMattDensity /= NbInd;
      RadialDist = LG_SLICE + LG_SLICE * Ind;
      RadialDist /= PLOT_DIST_UNIT;
      fprintf(pFile, "%f %f\n", RadialDist, SurfMattDensity );
    }
  }
  else ;    
}

void PlotMeanTanMeanFile(FILE *pFile)
{
  int Ind, NbInd, i;
  double Speed, RadialDist;
  double tmp = LG_SLICE;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      Speed = 0;
      NbInd = 0;
      for(i = -R_MEAN; i <= R_MEAN ; i++)
      {
        if(Ind+i >= 0)
        {
          Speed += SigmaTangentSpeed[Ind+i].Mean;
          NbInd++;
        }
        else ;
      }
      if(NbInd == 0) Error(2001);
      else Speed /= NbInd;
      RadialDist = LG_SLICE + LG_SLICE * Ind;
      RadialDist /= PLOT_DIST_UNIT;
      fprintf(pFile, "%f %f\n", RadialDist, Speed );
    }
  }
  else ;    
}

/* -------------- Testing GetCell code in Unit Test ---------------- */
void TestStat()       // Teste OK le 29 sept 2014.
{
  double Pos[3], Incr;
  unsigned int NumCell[3];
  unsigned int NbIn=0, NbOut=0, NbP=0, CR;
  double tmp1, tmp2, tmp3;
    
  PosOrigin[0] = 120 * LG_GRID;
  PosOrigin[1] = PosOrigin[2] = -7879.7 * LG_GRID;

  tmp1 = LG_GRID/2;
  tmp2 = LGX_CELL;
  tmp3 = LG_GRID/LGX_CELL;
  Incr = LGX_CELL/1;
  for( Pos[0] = PosOrigin[0]-LG_GRID/2 ; Pos[0] <= PosOrigin[0]+LG_GRID/2; Pos[0]+=Incr)
  {
    for( Pos[1] = PosOrigin[1]-LG_GRID/2 ; Pos[1] <= PosOrigin[1]+LG_GRID/2; Pos[1]+=Incr)
    {
      NbP++;
      CR = GetCell(Pos, NumCell);
      if(CR == CELL_IN) NbIn++;
      else NbOut++;
    }
  }
  PosOrigin[0] = PosOrigin[1] = PosOrigin[2];    
}

#else    // INIT_TYPE != INIT_BULLET_CLUSTER
void StatSpeedCalc(SigmaSpeed_t SigmaSpeed[]){}
void StatCalc()                                {}
void PlotRadSigmaFile(FILE *pFile)               {}
void PlotTanSigmaFile(FILE *pFile)                 {}
void PlotTanMeanFile(FILE *pFile)                    {}
void PlotSurfacicMatterDensityFile(FILE *pFile)        {}
void PlotMeanSurfacicMatterDensityFile(FILE *pFile)      {}
void PlotMeanTanMeanFile(FILE *pFile)                      {}
void TestStat(){}     // Teste OK le 29 sept 2014.
particle_stat_t PartStat[1];   // For generating only.
#endif    // INIT_TYPE != INIT_BULLET_CLUSTER
