/*************************************** PARTICLE S STAT STRUCTURE **************************************/
typedef struct{
  double SpeedRadial;
  double SpeedTangent;
	double PosNorm;       // Distance of the particle from the centroid of the galaxy.
}particle_stat_t;

/*************************************** CALCULATION OF SIGMA function **********************************/
/* ------ number and positions of the slices of the ray ------- */
#define NB_SLICE_PERGRID      200                             // Nb of slices in a grid ray.
#define SLICE_NBGRID          24                              // Lg of last slice given in nb of grid ray. 
#define NB_SLICE              (SLICE_NBGRID * NB_SLICE_PERGRID)        // Total nb of slices in a ray.
#define LG_SLICE              ((SLICE_NBGRID * LG_GRID / NB_SLICE) /2) // "/2" because only a ray is plotted. 
#define R_MEANSURF            7                                        
#define R_MEAN                5                                        // Mean curve greater than 11.
/* -------------------------------------- radial slice structure -------------------------------------- */
typedef struct          // Speed is radial speed OR tangent speed.
{
  double Mean;          // Mean = Mean value of speed in this slice.
  double S2;            // sigma = Square_root_of( (SIGMA of (Speed - Mean)) / NbPart )
  unsigned int NbPart;  // Nb particles in this slice.
}SigmaSpeed_t;

/******************************************* GLOBAL VARIABLES *******************************************/
/* ----------------------------------------- Engine variables ----------------------------------------- */
#if INIT_TYPE == INIT_BULLET_CLUSTER
  extern particle_stat_t PartStat[1];   // For generating only.
#else
  extern particle_stat_t PartStat[NBMAX_PART];
#endif
extern SigmaSpeed_t SigmaRadialSpeed[NB_SLICE];
extern SigmaSpeed_t SigmaTangentSpeed[NB_SLICE];

/***************************************** MACROS and FUNCTIONS *****************************************/
extern void StatCalc();
extern void PlotRadSigmaFile(FILE *pFile);
extern void PlotTanSigmaFile(FILE *pFile);
extern void PlotTanMeanFile(FILE *pFile);
extern void PlotSurfacicMatterDensityFile(FILE *pFile);
extern void PlotMeanSurfacicMatterDensityFile(FILE *pFile);
extern void PlotMeanTanMeanFile(FILE *pFile);

