/*   STET's modification of NEWTON's law.
    This file handles each function dealing closely or not with STET model.
*/

#include <stdlib.h> 
#include <string.h> 
#include <stdio.h>
#include <math.h>
#include "common.h"
#include "init.h"
#include "particles.h"
#include "fft.h"
#include "bulletcluster.h"
#include "stet.h"

#if GRAVIT_MODEL==STET           // Corrresponding #else is at the end of file.
  unsigned int StetNbPartInit;   // Nb of particles in a STET disk at init time.
  double StetAddedNbPart;
  double StetAddedDensity;
  double StetSolSystContrib;
  double StetAddedContrib;
  double StetIGMContrib;
  unsigned char StetDisk[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
  double StetCorrMax, StetCorrMin;
  #if STET_MODE == STET_COMPLEX || STET_MODE == STET_BULLET
    double StetR[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
    double StetCorrMemo[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
  #endif
  #if STET_MODE == STET_COMPLEX
    unsigned int StetNbPart[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];
  #endif

/* ------------------------ STET INIT ----------------------- */
void ReInitStet()
{
  StetCorrMax = 0;
  StetCorrMin = 10000000;
}

void InitStet()
{
	unsigned int jx, jy, jz;
  double PartMass0;
    
  StetCorrMax = 0;
  StetCorrMin = 10000000;
  #if STET_MODE == STET_COMPLEX
    double rho; 
    int i;
    for(i=0 ; i<LGMAX_STETR ; i++) *( ((double *)StetR) + i) = R0_GAL_AROUND_SUN;    //STET_R_MAX;
  #endif 

  /* ------- Init of StetNbPartInit, corresponds to local mass density */
  if(MODE_DEBUG == DEBUG_SOLSYST)
  {
    StetNbPartInit = NbPart;
  }
  else
  {
    #if NB_DIM == 3
      //       StetNbPartInit = (unsigned int) ((2*STET_R_MAX/LG_GRID) * (2*STET_R_MAX/LG_GRID) * (2*STET_R_MAX/LG_GRID) * NbPart);
      StetNbPartInit = (unsigned int) ((4/3)*PI*((STET_R_MAX*STET_R_MAX*STET_R_MAX) / (LG_GRID*LG_GRID*LG_GRID)) * NbPart);
    #else                                // NB_DIM == 2
      #if STET_MODE == STET_COMPLEX 

        double tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, tmp7, tmp8, tmp9, tmp10;

        //    #define R0_GAL_AROUND_SUN           (LN_2 / (SIGMA_EXT * NO_GAL_AROUND_SUN))    // in m.
        //    #define STET_DISK_SURF              (PI * R0_GAL_AROUND_SUN * R0_GAL_AROUND_SUN)

        tmp1 = NO_GAL_AROUND_SUN;
        tmp2 = RHO_GAL_AROUND_SUN;
        tmp3 = STET_DISK_SURF;
        tmp4 = FINAL_GAL_RAY;
        tmp5 = FINAL_GAL_SURF;
        tmp6 = STET_DISK_SURF / FINAL_GAL_SURF;
        tmp7 = RHO_GAL_AROUND_SUN; 
        tmp8 = MU0; 
        tmp10 = GALACTIC_WIDTH;

        StetNbPartInit = (unsigned int) (NbPart * STET_DISK_SURF / FINAL_GAL_SURF);  // yields 121539 with recup values...
        StetSolSystContrib = StetNbPartInit / PI;   // = R rho = R M/(PI R) = M/PI, 
                       // and PartMass not here bec its simplified in final result.
            // Here StetSolSystContrib is correct assuming that 
            // MU0 (=MU0HOHL) = RHO_GAL_AROUND_SUN. 
        rho = MU0 / GALACTIC_WIDTH;
        tmp9 = RHO_GAL_AROUND_SUN / rho; 

        // Peut etre faire un petit prog qui calcule Rho0 ou plutot StetCorr=1 a StetNbPartInit.
        // avant cela partir plutot d une etat de convergence Newton.
        // Et aller progressivement vers un etat de conv stet tel que stetnbpartinit corresponden a stetcorr=1.
        StetSolSystContrib *= RHO_GAL_AROUND_SUN / rho;   // cause hohl gal is less dense (1/2) and smaller (1/2) and lighter (1/8) than mw.
        StetSolSystContrib *= 3;    // FITTED 
        StetSolSystContrib *= 1.E3;    // FITTED 
        StetSolSystContrib *= 2;    // FITTED 
              // final result : StetSolSystContrib = (Mu0 / GalacticWidth) X 30
              // 
              // Le but est de calculer le nb de particules presentes dans un disque Stet de rayon 
              // N0 (celui du sol syst), correspondant a une contribution egale a celle du sol syst,
              // donc a une densite volumique egale  Rho0 (la densite volumique du sol syst).
              // 
              // La densite volumique de la galaxie HOHL est deja en general 2 fois plus faible 
              // que celle de la MW. En effet, le rayon 8 kpc de HOHL est 2 fois plus faible que celui
              // MW, et sa masseest environ 8 fois plus faible. Donc la densite volumique moyenne varie 
              // en 2/8 = 1/2.
              // Ceci ramene donc a 15 le coefficient 30 ci-dessus. 15 peut facilement s expliquer par
              // un Ngas different de celui de MW, quand Rho = Rho0, il suffit pour cela un ratio R/R0 
              // (ratio de StetRReal/R0) de sqrt(15) soit environ 4, donc le meme ratio sur le 
              // gas reel HOHL par rapport a celui de la galaxie MW, tous les deux pour le meme
              // Rho = Rho0.
              // De plus, GalacticWitdh utilise ci-dessus n est pas correct, car la densite
              // volumique n 'est pas constante le long de cette epaisseur de la galaxie.
              // En meme temps, ce GalacticWidth est celui de l'init de la galaxie HOHL,
              // qui est deja plus faible (8 kpc au lieu de 50 kpc, soit un ratio de 1/6) 
              // que celui de la galaxie du regime permanent 
              // obtenu lors des simulations. Ces 2 points se compensent donc grosierement.    
        StetAddedContrib = STET_ADDED_CONTRIB_RATIO_OVERSOLSYST * StetSolSystContrib;
        StetIGMContrib = STET_IGM_CONTRIB_RATIO_OVERSOLSYST * StetSolSystContrib;

        // simu 5 : ajouts prise en compte du "facteur de masse" de la galaxie.
        PartMass0 = TOTAL_MASS0 / NbPart; 
        StetAddedContrib *= PartMass0;                                     // For simu 5 but let that for all. 
        StetIGMContrib *= PartMass0;                                       // For simu 5 but let that for all. 
        StetSolSystContrib *= PartMass0;                                   // For simu 5 but let that for all.
      #elif STET_MODE == STET_SIMPLE 

        StetNbPartInit = (unsigned int) (NbPart * STET_DISK_SURF / FINAL_GAL_SURF);  // yields 121539 with recup values...

      #elif STET_MODE == STET_BULLET 

        InitStetBullet();

      #else  
      #endif 
    #endif
  }
  
  /* ------- Init of StetDisk, in which mass density is calculated */
	memset(StetDisk, 0, sizeof(StetDisk));
	for(jy=0 ; jy<NBCELLY_GRID ; jy++)
	{
		for(jx=0 ; jx<NBCELLX_GRID ; jx++)
		{
  		for(jz=0 ; jz<NBCELLZ_GRID ; jz++)
  		{
  			double x, y, z;
  			x = jx * LGX_CELL;
  			y = jy * LGY_CELL;
  			z = jz * LGZ_CELL;
  			if(x*x + y*y + z*z  <=  STET_R_MAX * STET_R_MAX)
  			{
  				StetDisk[jx][jy][jz] = 1;                 // This is inside the disk.
        } else ;                                    // Outside the disk.
      }
		}
	}
}

/* ----------------- Complex or Simplified STET ? ------------------- */
/* --------------------------- SIMPLE STET -------------------------- */
#if STET_MODE == STET_SIMPLE && INIT_TYPE != INIT_BULLET_CLUSTER 
  #define InsideStetDisk(ix, iy, iz, ax, ay, az)    (StetDisk[ax][ay][az])

  double GetStetCorrSimple(unsigned int NbPartInStet)
  {
    double StetCorr;

    // OLD VERSION
    //	StetCorr = StetNbPartInit;
    //	StetCorr /= NbPartInStet + STET_ADDED_NBPART;   // 1 because of the concerned particle itself....

    // NEW VERSION
    StetCorr = StetNbPartInit + STET_ADDED_NBPART;
    StetCorr /= NbPartInStet + STET_ADDED_NBPART;
//    StetCorr *= 6;   // simu 17 *3 c est trop, *2 pas mal mais bof, *1 ca dilue. Heuh non 3 cest le top. 4 dilu. Reprendre *3 et Rmax plus faible.
    StetCorr *= 3;   // simu 18

    #if (MODE_DEBUG == DEBUG_SOLSYST)             // Of course in sol syst no STET.
      //StetCorr = 1.;
    #else
      //    StetCorr *= 1.8;   // Pour corriger le cas STET_ADDED_NBPART = 50000
    #endif
  
    return(StetCorr);
  }
/* --------------------------- COMPLEX STET -------------------------- */
#elif STET_MODE == STET_COMPLEX && INIT_TYPE != INIT_BULLET_CLUSTER
  unsigned char InsideStetDisk(unsigned int ix, unsigned int iy, unsigned int iz, 
                              unsigned int ax, unsigned int ay, unsigned int az)
  {
    double x = ax * LGX_CELL;  
    double y = ay * LGY_CELL; 
    double z = az * LGZ_CELL; 
    return( x*x + y*y + z*z  <=  StetR[ix][iy][iz] * StetR[ix][iy][iz] );
  }

double tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, tmp7, tmp8, tmp9, tmp10, tmp11, tmp12, tmp13, tmp14, tmp15, tmp70, tmp700, tmp701, tmp702, tmp71, tmp72, tmp73, tmp74;    // PPOVISOIRE !!! pour debug !!!!!!!!!!!!!!!!

  double GetNGas(double Rho)   // PROVISOIRE, A CHANGER EN DENSITE SURFACIQUE PLUS TARD !!!!???????????????????????????
  {
    double Ngas, Ngalaxy; 

    tmp1 =  N_IGM;
    tmp2 =  NGAS_MAX;
    tmp3 =  RHO_A;
    tmp4 =  RHO_B;
    tmp5 =  DIFF_RHOAB;
    tmp6 = -(Rho-RHO_B)*(Rho-RHO_B)/SIGMA2R0; 
    tmp7 = exp(-(Rho-RHO_B)*(Rho-RHO_B)/SIGMA2R0); 
    tmp8 = -(Rho-RHO_B)*(Rho-RHO_B)/SIGMA2R0; 
    tmp9 = exp(-(Rho-RHO_B)*(Rho-RHO_B)/SIGMA2R0); 
    tmp10 = 1 - exp(-(Rho-RHO_B)*(Rho-RHO_B)/SIGMA2R0); 
    tmp11 = 1 - exp(-DIFF_RHOAB*DIFF_RHOAB/SIGMA2R0); 
    tmp12 = NGAS_MAX * exp(-(Rho-RHO_B)*(Rho-RHO_B)/SIGMA2R0); 
    tmp14 = -(Rho-RHO_B); 
    tmp15 = -(Rho-RHO_A)*(Rho-RHO_A)/SIGMA2R0; 
    tmp13 = NO_GAL_AROUND_SUN;

    if(Rho < RHO_A)
    {
      Ngalaxy = 0;
    }
    else if(Rho > RHO_B)
    {
      Ngalaxy = NGAS_MAX;
    }
    else
    {
      Ngalaxy =  1 - exp(-(Rho-RHO_A)*(Rho-RHO_A)/SIGMA2R0);
      Ngalaxy /= 1 - exp(-DIFF_RHOAB*DIFF_RHOAB/SIGMA2R0); 
      Ngalaxy *= NGAS_MAX * exp(-(Rho-RHO_B)*(Rho-RHO_B)/SIGMA2R0); 
    }
    Ngas = N_IGM + Ngalaxy;                           
    return( Ngas );   
  }                                  

  double GetNGasOld(double Rho)   // PROVISOIRE, A CHANGER EN DENSITE SURFACIQUE PLUS TARD !!!!???????????????????????????
  {
    double Ngas, K, Ngalaxy; 

    tmp1 =  N_IGM;
    tmp2 =  NGAS_MAX;
    tmp3 =  PI/2 - atan(1.E300) + atan((PI/(2*NGAS_MAX)) * (NO_GAL_AROUND_SUN / RHO_GAL_AROUND_SUN) * Rho);
    
    tmp4 =  NGAS_MAX;
    tmp5 =  N_IGM;
    tmp6 =  RHO_LOWDELTA;
    tmp7 =  atan((PI/RHO_LOWDELTA) * (Rho - RHO_LIMIT));
    tmp70 =  -PI/2;
    tmp700 =  1E-30;
    tmp701 =  atan(tmp700);
    tmp702 =  atan(-1.E290);
    tmp71 =  -PI/2 - atan(-1.E290) + atan((PI/RHO_LOWDELTA) * (Rho - RHO_LIMIT));
    tmp73 =  ((NGAS_MAX - N_IGM)/PI) * (-PI/2 - atan(-1.E290) + atan((PI/RHO_LOWDELTA) * (Rho - RHO_LIMIT)));
    tmp74 =  ((NGAS_MAX - N_IGM)/PI) * (-PI/2 - atan(-1.E290) + atan((PI/RHO_LOWDELTA) * (Rho - RHO_LIMIT))) + (NGAS_MAX + N_IGM)/2;
    tmp8 =  ((NGAS_MAX - N_IGM)/PI) * atan((PI/RHO_LOWDELTA) * (Rho - RHO_LIMIT));
    tmp9 =  -((NGAS_MAX - N_IGM)/2);
    tmp10 =  tmp9 + tmp3;

    // This yields very low accel at the end of the galaxy, brutally icreasing
    // at the "very end" (2 particles in StetDisk). 
    K = 2 * NO_GAL_AROUND_SUN / (RHO_GAL_AROUND_SUN * NGAS_MAX); 
                     // NO_GAL_AROUND_SUN = roughtly GetNGas(RHO_GAL_AROUND_SUN)
    Ngalaxy = 2 * NGAS_MAX / (1 + exp(-K*Rho)) - NGAS_MAX; 
    Ngas = N_IGM + Ngalaxy;                           
    
    return( Ngas );   
  }                                  

  double GetNGasNew(double Rho)    
  {
    double Ngas, Ngalaxy, Rho1, RhoRatio; 

    tmp1 =  N_IGM;
    tmp2 =  NGAS_MAX;
    tmp3 =  NGAS_ALPHA;
    tmp4 =  NGAS_GAMMA;
    tmp4 =  NGAS_K;

    Rho1 = pow(NGAS_MAX/NO_GAL_AROUND_SUN - 1, 1/NGAS_ALPHA); 
    Rho1 *= NGAS_GAMMA * RHO_GAL_AROUND_SUN; 
    RhoRatio = pow(Rho1/Rho, NGAS_ALPHA);   // A VERIFIER.
    Ngalaxy = NGAS_MAX / (1 + RhoRatio * exp(-NGAS_K*Rho)); 
    Ngas = N_IGM + Ngalaxy;                           
    
    return( Ngas );   
  }                                  

  // BEFORE this call: Calculation of NbPartInStet in Stet sphere of RAY equal
  // to StetR[ix][iy][iz], which has been calculated the step before in this 
  // function (just below). 
  double GetStetCorrComplex(unsigned int NbPartInStet, unsigned int ix, unsigned int iy, unsigned int iz) 
  {
    double StetCorr, NGas, StetRext, Stetr, StetNbPartSurfDensity, StetDensity, Contrib;

    StetNbPartSurfDensity = NbPartInStet;
    StetNbPartSurfDensity /= PI * StetR[ix][iy][iz] * StetR[ix][iy][iz];
    StetDensity = PartMass * StetNbPartSurfDensity / GALACTIC_WIDTH;
    NGas = GetNGas(StetDensity);
    StetRext = LN_2 / (SIGMA_EXT * NGas);
    Stetr = min(StetRext, STET_R_MAX);                       // Max STET_R_MAX=E21
    Contrib = StetNbPartSurfDensity * Stetr * Stetr;         // Contrib = NbPartInStet / PI
    Contrib *= PartMass;                                     // For simu 5 but let that for all. 
    Contrib += StetIGMContrib;                  // Out of the galaxy its not void, but IGM.
    StetCorr  =  StetSolSystContrib + StetAddedContrib;      // 
    StetCorr /=  Contrib            + StetAddedContrib;      // 

    StetCorrMemo[ix][iy][iz] = StetCorr;
    StetR[ix][iy][iz] = Stetr;
    StetNbPart[ix][iy][iz] = NbPartInStet;  // DEBUG only.

 if(StetCorr < 0)
 {
   return(StetCorr);
 }

// StetCorr = 1.;
    return(StetCorr);
  }
#else    // Nothing there. 
#endif 

double GetStetCorr(unsigned int NbPartInStet, unsigned int ix, unsigned int iy, unsigned int iz)
{
  #if STET_MODE == STET_SIMPLE 
    return( GetStetCorrSimple(NbPartInStet) );
  #elif STET_MODE == STET_COMPLEX
    return( GetStetCorrComplex(NbPartInStet, ix, iy, iz) );
  #elif STET_MODE == STET_BULLET
    return( GetStetCorrBullet(NbPartInStet, ix, iy, iz) );
  #else 
  #endif 
}

double GetStetCorrOut(unsigned int NumCell[])
{
  #if STET_MODE == STET_SIMPLE 
    return( GetStetCorrSimple(0) );
  #elif STET_MODE == STET_COMPLEX 
    return( StetCorrMemo[NumCell[0]][NumCell[1]][NumCell[2]] );
  #else 
    return( 1 );          // Never executed.
  #endif 
}

void GetAccelFromGalaxyStet(int PartInd, double accel[], unsigned int NumCell[])
{
	double StetCorr;        // Corrective term added to Newton's acceleration.

  /* --------------------- Calculate corrective term ---------------------- */
  StetCorr = GetStetCorrOut(NumCell);

  /* ---------------- Muliply accelerations by corrective term ------------ */
	accel[0] *= StetCorr;
	accel[1] *= StetCorr;
  #if (NB_DIM == 3)
	  accel[2] *= StetCorr;
  #endif
}

void PrintStetInfo(FILE *pFile)
{
    double RMaxRat = STET_R_MAX/LG_GRID;
    fprintf(pFile, " MinMaxC=%f,%f, RMaxR=%f", StetCorrMin, StetCorrMax, RMaxRat);           // OK
}

/* -------------- Testing STET code in Unit Test ---------------- */
#define NBPART 10000000
#define NBI 1000
double StetC[NBI];
double RelRho[NBI];
double NGas[NBI];
double Rext[NBI];
double StetC2_[NBI];
double RelRho2[NBI];
double NGas2[NBI];
double Rext2[NBI];
          //double AddedNbPart;          //                                  50000
double RhoGalAroundSun;      //                               7E-21 kg/m3
double NMax;                 // 20099999999.99 =               2E10 1/m3
double RatioNRho;            // 9.4632768361581904e+029 1/kg = e+30 1/kg
double StetRMax;             // 8.3160000000000000e+020 =       26 kpc.    
double R0GalAroundSun;       // 1.034e+018 =                    103 pc.
double N0GalAroundSun;       // 6699999999.9
double StetDiskSurf;         // 2.1725934565103999e+042 m3 = 2.17e+042 m3   
double FinalGalSurf;         // 1.0986349133168639e+044    = 1.0986349133168639e+044 m3
//StetNbPartInit                                                 200900
double StetC0;               // 0.84864031827816044
double StetC2, StetC4, StetC8, StetC16, StetC32, StetC64;
double StetC_2, StetC_4, StetC_8, StetC_16, StetC_32, StetC_64;
double StetCmax;

//double StetC[10] : par paquets de 10000:

// With NGAS_MAX = 3 * NO_GAL_AROUND_SUN) first simu, dilution :
//		[0]	0.051718804577474797	double
//		[1]	1.0620974568912329	double
//		[2]	0.69803153027975995	double
//		[3]	0.50821215324839408	double
//		[4]	0.39798062646154686	double
//		[5]	0.32664017423840153	double
//		[6]	0.27684718687023196	double
//		[7]	0.24016743462578286	double
//		[8]	0.21204142214424154	double
//		[9]	0.18979733037661889	double
// And for StetNbPartInit : StetCorr = 0.8 approxim.

// With NGAS_MAX = 30 * NO_GAL_AROUND_SUN) first simu, dilution :
//		[0]	0.051718804577474797	double
//		[1]	4.7718985066677480	double
//		[2]	8.4713273096908779	double
//		[3]	10.766151499020399	double
//		[4]	11.912928187781047	double
//		[5]	12.305504674512719	double
//		[6]	12.256190518024068	double
//		[7]	11.966753257568243	double
//		[8]	11.557073446391533	double
//		[9]	11.095716139446010	double
// And for StetNbPartInit : StetCorr = 1 approxim.

void TestStet()
{
//  double StetCorr;
//  unsigned char Inside;
//  unsigned int NbPart;
//  unsigned int ix, iy, iz, ax, ay, az;

//  NbPart = 0;    
//  ix = 0;    
//  iy = 0;    
//  iz = 0;    
//  ax = 0;    
//  ay = 0;    
//  az = 0;    
//  StetCorr = GetStetCorr(NbPart, ix, iy, iz);
//  Inside = InsideStetDisk(ix, iy, iz, ax, ay, az);

  //NbPart = 100000;    
  //InitStet();

  #if STET_MODE == STET_COMPLEX 
    RhoGalAroundSun =  RHO_GAL_AROUND_SUN;                  // Order of mag E-20.
    NMax = NGAS_MAX;
    RatioNRho = RATIO_N_RHO;      
    StetRMax = STET_R_MAX;
    R0GalAroundSun = R0_GAL_AROUND_SUN;
    N0GalAroundSun = NO_GAL_AROUND_SUN;
    StetDiskSurf = STET_DISK_SURF;
    FinalGalSurf = FINAL_GAL_SURF;
             //    StetNbPartInit = (unsigned int) (NbPart * STET_DISK_SURF / FINAL_GAL_SURF);
    NGas[0] = GetNGas(0);
    NGas[1] = GetNGas(RHO_GAL_AROUND_SUN);
    NGas[2] = GetNGas(1.0E50 * RHO_GAL_AROUND_SUN);
    NGas[3] = GetNGas(RHO_A);
    NGas[4] = GetNGas(RHO_B);
    StetC0 = GetStetCorrComplex(StetNbPartInit, 0,0,1);
    StetC2 = GetStetCorrComplex(StetNbPartInit*2, 0,0,2);
    StetC4 = GetStetCorrComplex(StetNbPartInit*4, 0,0,3);
    StetC8 = GetStetCorrComplex(StetNbPartInit*8, 0,0,4);
    StetC16 = GetStetCorrComplex(StetNbPartInit*16, 0,0,5);
    StetC32 = GetStetCorrComplex(StetNbPartInit*32, 0,0,6);
    StetC64 = GetStetCorrComplex(StetNbPartInit*64, 0,0,7);
    StetC_2 = GetStetCorrComplex(StetNbPartInit/2, 0,0,8);
    StetC_4 = GetStetCorrComplex(StetNbPartInit/4, 0,0,9);
    StetC_8 = GetStetCorrComplex(StetNbPartInit/8, 0,0,10);
    StetC_16 = GetStetCorrComplex(StetNbPartInit/16, 0,0,11);
    StetC_32 = GetStetCorrComplex(StetNbPartInit/32, 0,0,12);
    StetC_64 = GetStetCorrComplex(StetNbPartInit/64, 0,0,13);
    StetCmax = GetStetCorrComplex(0,0,0,14);    // Should get Zwicki like correction (5 - 10)

    // printing first this N = N(rho), and 2nd Rext = Rext(radialdist).
    // printing stetcorr = stetcorr(NbPartStetGas)
    for(int i = 0; i<NBI ; i++)
    {
      int NbParti;
      double rho;
      int NbPartInit;
      
      NbParti = i * (NBPART/NBI);
      NbPartInit = NbPart * 0.02 / 6;                               //2/50
      rho = RHO_GAL_AROUND_SUN * ((double) NbParti) / NbPartInit;
      RelRho[i] = rho/RHO_GAL_AROUND_SUN;
      NGas[i] = GetNGas(rho);
      StetC[i] = GetStetCorrComplex(NbParti, 0,0,0);
      Rext[i] = LN_2/(SIGMA_EXT * NGas[i]);
    }

    for(int i = 0; i<NBI ; i++)
    {
      int NbParti;
      double rho;
      int NbPartInit;
      
      NbParti = i * (NBPART/NBI);
      NbParti /= 1000;
      NbPartInit = NbPart * 0.02;                               //2/50
      rho = RHO_GAL_AROUND_SUN * ((double) NbParti) / NbPartInit;
      RelRho2[i] = rho/RHO_GAL_AROUND_SUN;
      NGas2[i] = GetNGas(rho);
      StetC2_[i] = GetStetCorrComplex(NbParti, 0,0,0);
      Rext2[i] = LN_2/(SIGMA_EXT * NGas2[i]);
    }
  #endif 
}

#else  // NEWTON case (for linking only)

  void InitStet(){}
  void ReInitStet(){}
  void GetAccelFromGalaxyStet(int PartInd, double accel[], unsigned int NumCell[]){}
  double GetStetCorr(unsigned int NbPartInStet, unsigned int ix, unsigned int iy, unsigned int iz){return(0);}
  void PrintStetInfo(FILE *pFile){}

#endif
